package com.wizarpos.mdbtest;

import android.content.Context;
import android.os.Bundle;
import android.os.SystemClock;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.wizarpos.util.MDBUtils;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashSet;
import java.util.List;

public class HomeFragment extends Fragment implements View.OnClickListener{

	private Button openBtn, closeBtn, btnGetVersion, btnUpdateFw, btnFactoryMode,
				   btnTest, btnGetHardwareVersion, btnDiagnoseHardware, btnTestVmc;
	private Spinner spnWhiteDemon;
	private String spnItemWhiteDemon;
	private Context context;

	private DataSendListener dataSendListener;
	private volatile String fileNameEnd = "";
	private List<String> spnNames = new ArrayList<>();
	private String[] ftpFileList = null;

	private static final String TAG = "HomeFragment";
	private static final String FTP_SERVER = "192.168.200.13";
	private static final String FTP_USER = "mdb_ro";
	private static final String FTP_PASSWD = "]Fm?jyJXn+{M&kq6mTrogFk";
	private static final String FTP_PATH = "STM32FWUpdate/";

	@Nullable
	@Override
	public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
		View view = inflater.inflate(R.layout.fragment_home, container, false);

		spnWhiteDemon = (Spinner) view.findViewById(R.id.spn_white_demon);

		openBtn = (Button) view.findViewById(R.id.open);
		closeBtn = (Button) view.findViewById(R.id.close);
		btnGetVersion = (Button) view.findViewById(R.id.btn_get_version);
		btnUpdateFw = (Button) view.findViewById(R.id.btn_update_fw);
		btnFactoryMode = (Button) view.findViewById(R.id.btn_factory_mode);
		btnTest = view.findViewById(R.id.btn_test);
		btnGetHardwareVersion = view.findViewById(R.id.btn_get_hardware_version);
		btnDiagnoseHardware = view.findViewById(R.id.btn_diagnose_hardware);
		btnTestVmc = view.findViewById(R.id.btn_test_vmc);

		openBtn.setOnClickListener(this);
		closeBtn.setOnClickListener(this);
		btnGetVersion.setOnClickListener(this);
		btnUpdateFw.setOnClickListener(this);
		btnFactoryMode.setOnClickListener(this);
		btnTest.setOnClickListener(this);
		btnGetHardwareVersion.setOnClickListener(this);
		btnDiagnoseHardware.setOnClickListener(this);
		btnTestVmc.setOnClickListener(this);

		context = getContext();
		updateSpinner();

		return view;
	}

	@Override
	public void onAttach(Context context) {
		super.onAttach(context);
		try{
			dataSendListener = (DataSendListener) context;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public void makeSpinner(List<String> binFiles){
		ArrayAdapter<String> adapterWhiteDemon = new ArrayAdapter<>(context, android.R.layout.simple_spinner_item, binFiles);
		adapterWhiteDemon.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spnWhiteDemon.setAdapter(adapterWhiteDemon);
		spnWhiteDemon.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			@Override
			public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
				spnItemWhiteDemon = spnWhiteDemon.getSelectedItem().toString();
				setFileName();
				if (fileNameEnd.isEmpty()) {
					Log.e(TAG, "FTP file not found");
				}
			}
			@Override
			public void onNothingSelected(AdapterView<?> adapterView) {

			}
		});
	}

	@Override
	public void onClick(View view) {
		switch (view.getId()) {
			case R.id.open:
				dataSendListener.onOpenClicked();
				break;
			case R.id.close:
				dataSendListener.onCloseClicked();
				break;
			case R.id.btn_get_version:
				dataSendListener.onGetVersionClicked();
				break;
			case R.id.btn_update_fw:
				dataSendListener.onIntValueSent(MDBUtils.DISABLE_ALL_UI, 0);
				SystemClock.sleep(1000);
				if(getFwVersion() <= 22){
					Log.d(TAG, "updating from assert");
					dataSendListener.onUpdateFwClicked();
				} else {
					Log.d(TAG, "updating from FTP");
					boolean fileExist = false;
					File filesDir = context.getFilesDir();
					if(filesDir.exists() && filesDir.isDirectory()){
						File[] files = filesDir.listFiles();
						if(files != null && files.length > 0){
							for(File file : files){
								if(file.getName().endsWith(fileNameEnd)){
									fileExist = true;
								}
							}
						}
					}
					if(fileExist) {
						Log.d(TAG, "firmware already downloaded to internal storage, downloading to mib");
					} else{
						Log.d(TAG, "downloading firmware from ftp...");
						downloadFileFTP();
						SystemClock.sleep(2000);
					}
					dataSendListener.onUpdateFwClicked();
				}
				break;
			case R.id.btn_factory_mode:
				dataSendListener.onFactoryModeClicked();
				break;
			case R.id.btn_test:
				dataSendListener.onTestClicked();
				break;
			case R.id.btn_get_hardware_version:
				dataSendListener.onGetHardwareVersionClicked();
				break;
			case R.id.btn_diagnose_hardware:
				dataSendListener.onDiagnoseHardwareClicked();
				break;
			case R.id.btn_test_vmc:
				dataSendListener.onTestVmcClicked();
				break;
		}
	}

	public void setFileName(){
		int nVersionNo = Integer.parseInt(spnItemWhiteDemon);
		if(nVersionNo < 23){
			dataSendListener.onStringValueSent(MDBUtils.TYPE_SPN_FIRMWARE_ITEM, spnItemWhiteDemon);
			Log.d(TAG, "filename is set to " + spnItemWhiteDemon);
		} else {
			for (String s : spnNames) {
				if (s.equals(spnItemWhiteDemon)) {
					fileNameEnd = nVersionNo + ".bin";
				}
			}
			dataSendListener.onStringValueSent(MDBUtils.TYPE_SPN_FIRMWARE_ITEM, spnItemWhiteDemon);
			Log.d(TAG, "filename is set to " + fileNameEnd);
		}

	}

	public int getFwVersion(){
		return Integer.parseInt(spnItemWhiteDemon);
	}

	public void updateSpinner(){
		new Thread(new Runnable() {
			@Override
			public void run() {
				FTPClient ftpClient = new FTPClient();
				String[] spnNamesResource = context.getResources().getStringArray(R.array.white_demon);
				spnNames.addAll(Arrays.asList(spnNamesResource));
				try {
					ftpClient.connect(FTP_SERVER);
					ftpClient.login(FTP_USER, FTP_PASSWD);
					ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
					ftpFileList = ftpClient.listNames(FTP_PATH);
					if(ftpFileList != null) {
						for (String s : ftpFileList) {
							if (s.length() == 42 && s.startsWith("STM32FWUpdate/WhiteDemon") && s.toLowerCase().endsWith(".bin")) {
								// STM32FWUpdate/WhiteDemon_8df8b7b6_0023.bin
								spnNames.add(s.substring(34, 38));
							} else {
								Log.w(TAG, "unknown file exists: " + s);
							}
						}
					}
					ftpClient.logout();
					ftpClient.disconnect();
				}catch (IOException e) {
					dataSendListener.onLogSent('e', TAG, "FTP connection failed!");
//					throw new RuntimeException(e);
				}
				spnNames = getSortedSpnNames(spnNames);
				getActivity().runOnUiThread(new Runnable() {
					@Override
					public void run() {
						makeSpinner(spnNames);
					}
				});
			}
		}).start();
	}

	public void downloadFileFTP(){
		new Thread(new Runnable() {
			@Override
			public void run() {
				updateSpinner();
				SystemClock.sleep(500);
				FTPClient ftpClient = new FTPClient();
				InputStream inputStream = null;
				FileOutputStream outputStream = null;
				String savePath = context.getFilesDir().getPath();
				try {
					ftpClient.connect(FTP_SERVER);
					boolean logIn = ftpClient.login(FTP_USER, FTP_PASSWD);
					if(ftpClient.isConnected() && logIn) {
						Log.d(TAG, "connect to FTP success");
						ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
						String fileName = "";
						for (String s : ftpFileList) {
							if (s.toLowerCase().endsWith(fileNameEnd)) {
//							fileName = s.substring(14);
								//WhiteDemon_8df8b7b6_0023.bin
								if (s.substring(14).length() == 28) {
									fileName = s.substring(14);
									Log.d(TAG, "fileName: " + fileName);
								}
							}
						}
						File outputFile = new File(savePath, fileName);
						inputStream = ftpClient.retrieveFileStream(FTP_PATH + fileName);
						if (inputStream != null) {
							Log.d(TAG, "downloading file: " + FTP_PATH + fileName);
							outputStream = new FileOutputStream(outputFile);
							byte[] readBuf = new byte[4096];
							int bytesRead;
							while ((bytesRead = inputStream.read(readBuf)) != -1) {
								outputStream.write(readBuf, 0, bytesRead);
							}
							Log.d(TAG, "download firmware from FTP success");
							if (checkFileHash(outputFile, fileName.substring(11, 19))) {
								Log.d(TAG, "check file hash success");
								dataSendListener.onLogSent('d', TAG, "download file success");
							} else {
//							fileNameEnd = "";
								if (outputFile.delete()) {
									dataSendListener.onLogSent('e', TAG, "download file failed!");
									Log.e(TAG, "download file failed, file deleted!");
								} else {
									Log.e(TAG, "download file failed, delete file failed!");
								}
							}
							outputStream.close();
							inputStream.close();
						}
						ftpClient.logout();
						ftpClient.disconnect();
					} else {
						dataSendListener.onLogSent('e', TAG, "connection to FTP failed!");
					}
				} catch (IOException | NoSuchAlgorithmException e) {
					dataSendListener.onLogSent('e', TAG, "FTP connection failed!");
//					throw new RuntimeException(e);
				} finally {
					try {
						if(outputStream != null){
							outputStream.close();
						}
						if(inputStream != null){
							inputStream.close();
						}
					} catch (IOException e) {
						throw new RuntimeException(e);
					}
				}
			}
		}).start();
	}

	public boolean checkFileHash(File file, String fileHash) throws NoSuchAlgorithmException, IOException {
		MessageDigest messageDigest = MessageDigest.getInstance("SHA-256");
		FileInputStream fis = new FileInputStream(file);
		byte[] buffer = new byte[1024];
		int bytesRead = 0;
		while((bytesRead = fis.read(buffer)) != -1){
			messageDigest.update(buffer, 0 ,bytesRead);
		}
		byte[] hashBytes = messageDigest.digest();
		//transfer each byte to 2byte hex
		StringBuilder hexString = new StringBuilder();
		for(byte b : hashBytes){
			hexString.append(String.format("%02x", b));
		}
		String calculatedHash = hexString.toString();
		Log.d(TAG, "calculated hash: " + calculatedHash);
		if(fileHash.equals(calculatedHash.substring(0, 8))){
			return true;
		}
		return false;
	}

	public List<String> getSortedSpnNames(List<String> list){
		List<String> ret = new ArrayList<>();
		for(String s : list){
			ret.add(s.substring(0, 4));
		}

		int length = ret.size();
		for(int i = 0; i < length; ++i){
			for(int j = 0; j < length - 1; ++j){
				if(Integer.parseInt(ret.get(j)) < Integer.parseInt(ret.get(j + 1))){
					String tmp = ret.get(j);
					ret.set(j, ret.get(j + 1));
					ret.set(j + 1, tmp);
				}
			}
		}
		LinkedHashSet<String> set = new LinkedHashSet<>(ret);
		ret.clear();
		ret.addAll(set);
//		for(String s : ret){
//			Log.d(TAG, "ret: " + s);
//		}
		return ret;
	}

}
