package com.wizarpos.mdbtest;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.os.SystemClock;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.ActionBarDrawerToggle;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.view.GravityCompat;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;

import com.cloudpos.DeviceException;
import com.cloudpos.OperationListener;
import com.cloudpos.OperationResult;
import com.cloudpos.POSTerminal;
import com.cloudpos.TimeConstants;
import com.cloudpos.card.Card;
import com.cloudpos.jniinterface.SerialPortInterface;
import com.cloudpos.jniinterface.Stm32ispInterface;
import com.cloudpos.rfcardreader.RFCardReaderDevice;
import com.cloudpos.rfcardreader.RFCardReaderOperationResult;
import com.cloudpos.smartcardreader.SmartCardReaderDevice;
import com.google.android.material.navigation.NavigationView;
import com.wizarpos.util.ByteConvertStringUtil;
import com.wizarpos.util.EnumMDBCommands;
import com.wizarpos.util.LogHelper;
import com.wizarpos.util.MDBUtils;
import com.wizarpos.values.MDBValues;
import com.wizarpos.values.OptionalFeature;
import com.wizarpos.values.PulseValues;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.net.ServerSocket;
import java.net.Socket;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.Locale;
import java.util.Queue;

public class MainActivity extends AppCompatActivity implements DataSendListener {

    private DrawerLayout drawerLayout;
    private ActionBarDrawerToggle toggle;
    private TextView textView;

    private Handler handler;

    private MDBValues mdbValues = new MDBValues();
    private PulseValues pulseValues = new PulseValues();

    private Fragment selectedFragment = null;
    private volatile boolean running = true;

    public boolean readCardDone = true;
    public Queue<Message> msgQueueReadCard = new LinkedList<>();
    private Context context;

    private Card rfCard;
    private RFCardReaderDevice rfCardReaderDevice = null;
    private SmartCardReaderDevice smartCardReaderDevice = null;

    private final String TAG = "MDBTest TAG";
    private final boolean TRIGGER_PULSE_ONE_SHOT = true;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        textView = (TextView)findViewById(R.id.textviewaa);
        textView.setMovementMethod(ScrollingMovementMethod.getInstance());
        context = this;

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        drawerLayout = findViewById(R.id.drawer_layout);
        NavigationView navigationView = findViewById(R.id.nav_view);
        toggle = new ActionBarDrawerToggle(this, drawerLayout, toolbar, R.string.nav_drawer_open, R.string.nav_drawer_close);
        drawerLayout.addDrawerListener(toggle);
        toggle.syncState();
        selectedFragment = new HomeFragment();
//        getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, selectedFragment).commit();
        navigationView.setNavigationItemSelectedListener(new NavigationView.OnNavigationItemSelectedListener() {
            @Override
            public boolean onNavigationItemSelected(@NonNull MenuItem item) {
                if (item.getItemId() == R.id.nav_home) {
                    selectedFragment = getSupportFragmentManager().findFragmentByTag(HomeFragment.class.getSimpleName());
                    if (selectedFragment == null) {
                        selectedFragment = new HomeFragment();
                    }
                } else if (item.getItemId() == R.id.nav_pulse) {
                    selectedFragment = getSupportFragmentManager().findFragmentByTag(PulseFragment.class.getSimpleName());
                    if (selectedFragment == null) {
                        selectedFragment = new PulseFragment();
                    }
                } else if (item.getItemId() == R.id.nav_transaction) {
                    selectedFragment = getSupportFragmentManager().findFragmentByTag(TransactionFragment.class.getSimpleName());
                    if (selectedFragment == null) {
                        selectedFragment = new TransactionFragment();
                    }
                }
                FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();

                if(selectedFragment != null){
                    Fragment existingFragment = getSupportFragmentManager().findFragmentByTag(selectedFragment.getClass().getSimpleName());
                    if(existingFragment == null){
                        transaction.add(R.id.fragment_container, selectedFragment, selectedFragment.getClass().getSimpleName());
                    } else {
                        transaction.show(existingFragment);
                    }
                }
                for (Fragment fragment : getSupportFragmentManager().getFragments()) {
                    if (fragment != selectedFragment) {
                        transaction.hide(fragment);
                    }
                }
                transaction.commit();
                drawerLayout.closeDrawer(GravityCompat.START);
                return true;
            }
        });
        if(savedInstanceState == null){
            getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, selectedFragment).commit();
            navigationView.setCheckedItem(R.id.nav_home);
        }
        handler = new Handler(new Handler.Callback() {
            @Override
            public boolean handleMessage(Message msg) {
//                Log.d(TAG, "handleMessage: " + msg.what);
                switch (msg.what) {
                    case MDBUtils.ENABLE_ALL_UI:
                        enableAllUi();
                        break;
                    case MDBUtils.DISABLE_ALL_UI:
                        disableAllUi();
                        break;
                    case MDBUtils.BLACK_LOG:
                        LogHelper.appendBlackMsg((String) msg.obj, textView);
                        break;
                    case MDBUtils.BLUE_LOG:
                        LogHelper.infoAppendForAlert((String) msg.obj, textView);
                        break;
                    case MDBUtils.RED_LOG:
                        LogHelper.appendREDMsg((String) msg.obj, textView);
                        break;
                    case MDBUtils.GREEN_LOG:
                        LogHelper.appendGreenMsg((String) msg.obj, textView);
                        break;
                    case MDBUtils.DIALOG_CONFIRM:
                        Item itemDialog = (Item) msg.obj;
                        AlertDialog dialog = new AlertDialog.Builder(context)
                                .setTitle("Please confirm item and amount")
                                .setMessage("Vend product\n " + "Amount: $" + itemDialog.getItemPrice() + ",Item: " + itemDialog.getItemId())
                                .setNegativeButton("Deny", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        sendVendDedied();
                                        readCardDone = true;
                                        dialog.dismiss();
                                        processReadcard();
                                    }
                                })
                                .setPositiveButton("Approve", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        sendVendApproved(itemDialog);
                                        readCardDone = true;
                                        dialog.dismiss();
                                    }
                                }).create();
                        dialog.show();
                        break;
                    case MDBUtils.DIALOG_ITEM:
                        AlertDialog dialog1 = new AlertDialog.Builder(context)
                                .setTitle("Dispense")
                                .setMessage(msg.obj + "")
                                .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        processReadcard();
                                        dialog.dismiss();
                                    }
                                }).create();
                        dialog1.show();
                        break;
                    case MDBUtils.DIALOG_READCARD:
                        Message message = new Message();
                        message.what = msg.what;
                        message.obj = msg.obj;
                        msgQueueReadCard.add(message);
                        if(readCardDone) {
                            processReadcard();
                        }
                        break;
                    case MDBUtils.TEST_MSG:
                        if (rfCardReaderDevice == null) {
                            rfCardReaderDevice = (RFCardReaderDevice) POSTerminal.getInstance(context)
                                    .getDevice("cloudpos.device.rfcardreader");
                        }
                        try {
                            rfCardReaderDevice.open();
                            SystemClock.sleep(5000);
                            readCardDone = true;
                            rfCardReaderDevice.close();
                        } catch (DeviceException e) {
                            Log.e(TAG, "open rfCardReaderDevice failed !");
                            e.printStackTrace();
                        }
                        break;
                }
                return false;
            }
        });
        open();
    }

    @Override
    protected void onStop() {
        super.onStop();
        running = false;
        close();
    }

    @Override
    protected void onRestart() {
        super.onRestart();
        running = true;
        open();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        close();
    }

    public void processReadcard(){
        if(msgQueueReadCard.isEmpty()){
            readCardDone = true;
            return;
        }
        Log.d(TAG, "processing read card");
        readCardDone = false;
        Message msgReadCard = msgQueueReadCard.poll();
        AlertDialog cardDialog = new AlertDialog.Builder(context)
                .setTitle("Read card")
                .setMessage("Please swipe card.")
                .create();
        cardDialog.show();
        Item item = (Item) msgReadCard.obj;
//        Log.d(TAG, "Read item data " + item.getItemPrice() + "," + item.getItemId());
        if (rfCardReaderDevice == null) {
            rfCardReaderDevice = (RFCardReaderDevice) POSTerminal.getInstance(context)
                    .getDevice("cloudpos.device.rfcardreader");
        }
        if (smartCardReaderDevice == null) {
            smartCardReaderDevice = (SmartCardReaderDevice) POSTerminal.getInstance(context)
                    .getDevice("cloudpos.device.smartcardreader");
        }
        try {
            rfCardReaderDevice.open();
            try {
                OperationListener listener = new OperationListener() {
                    @Override
                    public void handleResult(OperationResult arg0) {
                        try {
                            if (arg0.getResultCode() == OperationResult.SUCCESS) {
                                rfCard = ((RFCardReaderOperationResult) arg0).getCard();
                                handler.obtainMessage(MDBUtils.BLACK_LOG, "amount: $" + item.getItemPrice() + ",item: " + item.getItemId()).sendToTarget();
                                handler.obtainMessage(MDBUtils.DIALOG_CONFIRM, item).sendToTarget();
                            } else {
                                Log.e(TAG, "find_card_failed");
                                readCardDone = true;
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        } finally {
                            try {
                                rfCardReaderDevice.close();
                                cardDialog.dismiss();
                            } catch (DeviceException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                };
                rfCardReaderDevice.listenForCardPresent(listener, TimeConstants.FOREVER);
            } catch (DeviceException e) {
                e.printStackTrace();
            }
        } catch (DeviceException e) {
            Log.e(TAG, "open rfCardReaderDevice failed !");
            e.printStackTrace();
        }
    }

    Thread thread =  new Thread(new Runnable() {
            @Override
            public void run() {
                Item item = new Item();
                handler.obtainMessage(MDBUtils.BLACK_LOG, "Init, waiting master command...").sendToTarget();
//                while (isSlave) {
                while (running) {
                    byte[] byteArr = new byte[1];
                    // read start flag 0x09, 1 bit
                    int readResult = SerialPortInterface.read(byteArr, byteArr.length, -1);
                    Log.d(TAG, "read 1 byte success: " + ByteConvertStringUtil.buf2StringCompact(byteArr));
                    if(ByteConvertStringUtil.buf2StringCompact(byteArr).equals("0D")){
                        continue;
                    }
                    if (readResult >= 0) {
                        if (ByteConvertStringUtil.buf2StringCompact(byteArr).equals("09")) {
                            // read data length, 1 bit
                            readResult = SerialPortInterface.read(byteArr, byteArr.length, 200);
                            if (readResult > 0) {
                                Log.d(TAG, "read length success: " + ByteConvertStringUtil.buf2StringCompact(byteArr));
                                String lenthHex = ByteConvertStringUtil.buf2StringCompact(byteArr);
                                BigInteger length = new BigInteger(lenthHex, 16);
                                // read data
                                byteArr = new byte[length.intValue()];
                                readResult = SerialPortInterface.read(byteArr, byteArr.length, 200);
                                if (readResult > 0) {
                                    Log.i(TAG, "read success: " + ByteConvertStringUtil.buf2StringCompact(byteArr));
                                } else {
                                    Log.e(TAG, "read failed");
                                }
                                processData(item, byteArr);
                            }
                            else {
                                Log.e(TAG, "read length failed!");
                            }
                        }
                    }
//                    else if (readResult == -262254) {
//                        handler.obtainMessage(MDBUtils.RED_LOG, "serialport read timeout").sendToTarget();
//                    } else {
//                        handler.obtainMessage(MDBUtils.RED_LOG, "serialport read failed = " + readResult).sendToTarget();
//                    }
                    if (   !compareByteArrayHead(byteArr, new byte[]{0x01, 0x20, 0x00}, 3)
                        && !compareByteArrayHead(byteArr, new byte[]{0x0D}, 1)) {
                        SystemClock.sleep(500);
                    }
                }
//                close();
            }
        });

    private void testMdbSlaveRead() {
        thread.start();
        running = true;
    }

    private void processData(Item item, byte[] readBytes){
        handler.obtainMessage(MDBUtils.BLUE_LOG, "read: ").sendToTarget();
        handler.obtainMessage(MDBUtils.BLACK_LOG, ByteConvertStringUtil.buf2StringCompact(readBytes)).sendToTarget();
        if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x10, 0x10}, 3)) {
            sendReset();
            mdbValues.setOptionalFeature(new OptionalFeature());
            CheckBox cbAlwaysIdle = selectedFragment.getView().findViewById(R.id.cb_always_idle);
            mdbValues.getOptionalFeature().setAlwaysIdleMdb(cbAlwaysIdle.isChecked());
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x11, 0x00}, 3)) { //Setup config data
            mdbValues.setVmcLevel(readBytes[3]);
            sendSetupConfig(item);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x11, 0x01}, 3)) { // Setup Max/Min Price
            sendSetupPrice();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x17, 0x00}, 3)) { //Expansion ID
            sendExpansionRequestID();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x17, 0x04}, 3)) { //Expansion optional feature bits
            mdbValues.getOptionalFeature().setOptionalFeatureBitsVmc(readBytes[6]);
            mdbValues.getOptionalFeature().setOptionalFeatureBitsMdb(readBytes[6]);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x14, 0x00}, 3)
                || compareByteArrayHead(readBytes, new byte[]{0x00, 0x14, 0x01}, 3)) {//Enable Reader
            sendEnableReader();
            startTransaction();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, 0x00}, 2)) {
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Ready! Please input item ID").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x15, 0x00}, 3)){ //Revalue approved
            sendRevalueApproved();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x15, 0x01}, 3)){ //Revalue limited
            sendRevalueLimit();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x13, 0x00}, 3)) { // vend request
            //e.g. 00 13 00 00 A0 00 01 B4 98
            //amount: 00 A0
            //id: 00 01
            if(mdbValues.getOptionalFeature().isMonetaryFormat32Vmc() && mdbValues.getOptionalFeature().isMonetaryFormat32Mdb())
            {
                item.setItemAmount(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[3], readBytes[4], readBytes[5], readBytes[6]}).replace(" ", ""));
                item.setItemId(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[7], readBytes[8]}).replace(" ", ""));
                item.calculatePrice();
                BigInteger itemid = new BigInteger(item.getItemId(), 16);
                item.setItemId(itemid.toString());
            } else {
                item.setItemAmount(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[3], readBytes[4]}).replace(" ", ""));
                item.setItemId(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[5], readBytes[6]}).replace(" ", ""));
                item.calculatePrice();
                BigInteger itemid = new BigInteger(item.getItemId(), 16);
                item.setItemId(itemid.toString());
            }
            handler.obtainMessage(MDBUtils.DIALOG_READCARD, item).sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x13, 0x02}, 3)) { //vend dispatch
            sendVendSuccess(readBytes);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x13, 0x04}, 3)) { //vend session end
            sendSessionEnd();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x13, 0x06}, 3)){ //negative vend request
            if(mdbValues.getMdbLevel() == 3 && mdbValues.getVmcLevel() == 3) {
                Item item1 = new Item();
                if(!mdbValues.getOptionalFeature().isMonetaryFormat32Vmc()) {
                    item1.setItemAmount(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[3], readBytes[4]}).replace(" ", ""));
                    item1.setItemId(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[5], readBytes[6]}).replace(" ", ""));
                    item1.calculatePrice();
                    BigInteger itemid = new BigInteger(item1.getItemId(), 16);
                    item1.setItemId(itemid.toString());
				} else {
                    item1.setItemAmount(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[3], readBytes[4], readBytes[5], readBytes[6]}).replace(" ", ""));
                    item1.setItemId(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[7], readBytes[8]}).replace(" ", ""));
                    item1.calculatePrice();
                    BigInteger itemid = new BigInteger(item1.getItemId(), 16);
                    item1.setItemId(itemid.toString());
				}
				sendNegativeVendApproved(readBytes, item1);
			}
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x14, 0x02}, 3)){ //reader cancel
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Reader cancelled").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x90, 0x03}, 3)) { //set duration
            handler.obtainMessage(MDBUtils.BLACK_LOG, "set pulse duration succeed").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x90, 0x04}, 3)) { //set voltage
            handler.obtainMessage(MDBUtils.BLACK_LOG, "set pulse voltage succeed").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x90, 0x05}, 3)){ //factory mode
            if(mdbValues.isFactoryMode()) {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "enter factory mode succeed").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "quit factory mode succeed").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x92, 0x00}, 3)) { //get version
            handler.obtainMessage(MDBUtils.BLACK_LOG, "get result:" +
                    ByteConvertStringUtil.buf2StringCompact(readBytes)).sendToTarget();
            printVersionName(readBytes);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x94}, 2)) { //diagnose hardware
            if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x94, 0x00}, 3)) {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "hardware diagnose test received").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "hardware diagnose test open failed").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x93}, 2)) { //get hardware version
            if(compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x93, 0x00}, 3)) {
              printHardwareVersion(readBytes);
            } else {
                handler.obtainMessage(MDBUtils.RED_LOG, "get hardware version failed!").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, (byte)0x13, 0x01}, 3)) {  // vend denied
            Log.d(TAG, "00 13 01, vending auto cancel, denied");
            sendVendDedied();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, 0x20, 0x00}, 3)) { //trigger pulse
            //receive: 09 04 01 20 00 DF 0D
            pulseValues.setCurrentSend(pulseValues.getCurrentSend() - 1);
            if (pulseValues.getCurrentSend() == 0) {
                handler.obtainMessage(MDBUtils.ENABLE_ALL_UI).sendToTarget();
                handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse " + pulseValues.getPulseFrequency() + " times succeed").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse " + (pulseValues.getPulseFrequency() - pulseValues.getCurrentSend())).sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x21}, 2)) { //trigger pulse one shot resp
            if(compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x21, 0x00}, 3)) {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse succeed").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.RED_LOG, "trigger pulse failed!").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x0d}, 1)){
            Log.d(TAG, "0D, end of data");
        } else {
            handler.obtainMessage(MDBUtils.BLACK_LOG, "read failed: " +
                    ByteConvertStringUtil.buf2StringCompact(readBytes)).sendToTarget();
        }
    }

    private boolean compareByteArrayHead(byte[] byteArray, byte[] bytesHead, int nBytes){
        byte[] tmpBytesHead = Arrays.copyOfRange(byteArray, 0, nBytes);
        return Arrays.equals(tmpBytesHead, bytesHead);
    }

    private void sendCommonResponse(){
        byte[] dataBytes = new byte[1];
        byte[] wbytes = MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "common response");
        }
    }
    private void sendReset(){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.RESET);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Reset wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("Reset", wbytes);
        }
    }
    //readBytes = "00 11 00 02 00 00 01 14"
    private void sendSetupConfig(Item item){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.SETUP_CONFIG, mdbValues, item);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Setup config data. wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("Setup config data", wbytes);
        }
    }
    private void sendSetupPrice(){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.SETUP_PRICE);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Setup Max/Min Price. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("Setup Max/Min Price", wbytes);
        }
    }
    private void sendExpansionRequestID(){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.EXPANSION_REQUESTID, mdbValues);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Expansion ID. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("Expansion ID", wbytes);
        }
    }
    private void sendEnableReader(){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.ENABLE_READER);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Enable Reader. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("Enable Reader", wbytes);
        }
    }
    private void sendRevalueApproved(){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.REVALUE_APPROVED);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Revalue approved. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("Revalue approved", wbytes);
        }
    }
    private void sendRevalueLimit(){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.REVALUE_LIMIT_AMOUNT);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Revalue limit request. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("Revalue limit request", wbytes);
        }
    }

    private void sendVendSuccess(byte[] readBytes){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.VEND_SUCCESS);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Vend success! wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            int itemIdHigh = readBytes[3];
            int itemIdLow = readBytes[4];
            int itemId = ((itemIdHigh & 0xFF) << 8) | (itemIdLow & 0xFF);
            handler.obtainMessage(MDBUtils.DIALOG_ITEM, "Vend success! " + ByteConvertStringUtil.buf2StringCompact(wbytes)
                    + " item:" + itemId).sendToTarget();
        }
    }
    private void sendSessionEnd(){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.END_SESSION);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "finish. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("finish", wbytes);
            SystemClock.sleep(2000);
            startTransaction();
        }
    }
    private void sendVendApproved(Item item){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.VEND_APPROVED, item);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "approved! wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("approved", wbytes);
        }
    }
    private void sendVendDedied(){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.VEND_DENIED);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "denied! wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("denied", wbytes);
        }
    }
    //not tested yet
    private void sendNegativeVendApproved(byte[] readBytes, Item item){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.NEGATIVE_VEND_APPROVED, mdbValues, item);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "negative vend approved! wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("negative vend approved", wbytes);
        }
    }

    //begin session
    private void startTransaction() {
        Log.d(TAG, "mdb: " + mdbValues.getOptionalFeature().isAlwaysIdleMdb() + ", vmc: " + mdbValues.getOptionalFeature().isAlwaysIdleVmc());
        if(mdbValues.getOptionalFeature().isAlwaysIdleMdb() && mdbValues.getOptionalFeature().isAlwaysIdleVmc()){
            Log.d(TAG, "always idle");
            handler.obtainMessage(MDBUtils.BLACK_LOG, "always idle").sendToTarget();
        } else {
            byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(EnumMDBCommands.MDB_REQUEST.BEGIN_SESSION, mdbValues);
            int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
            if (wresult >= 0) {
                Log.d(TAG, "begin session. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
                printLogWithBlueRead("begin session", wbytes);
            }
        }
    }
    private void sendReaderCancelled(String sp){
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.READER_CANCELLED);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "cancelled. wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            printLogWithBlueRead("cancelled", wbytes);
        }
    }
    private void sendTriggerPulse(){ //09 03 00 20 E0 0D resp: 09 04 01 20 00 DF 0D
        byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(EnumMDBCommands.MDB_REQUEST.TRIGGER_PULSE);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send trigger pulse succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
        }
    }
    private void sendTriggerPulseOneShot() {
        byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(EnumMDBCommands.MDB_REQUEST.TRIGGER_PULSE_ONESHOT, pulseValues);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send trigger pulse one shot succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
        }
    }
    private void sendSetPulseDuration(int duration){ //09 08 00 90 03 C8 00 00 00 A5 0D resp: 09 05 01 90 03 00 6C 0D
        byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(EnumMDBCommands.MDB_REQUEST.SET_PULSE_DURATION, duration);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send set pulse duration succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "setting pulse duration").sendToTarget();
        }
        SystemClock.sleep(200);
    }
    private void sendSetPulseVoltage(int voltage){ //09 08 00 90 04 00 00 00 00 6C 0D resp: 09 05 01 90 04 00 6B 0D
        byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(EnumMDBCommands.MDB_REQUEST.SET_PULSE_VOLTAGE, voltage);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send set pulse voltage succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "setting pulse voltage").sendToTarget();
        }
        SystemClock.sleep(200);
    }
    private void sendEnterFactoryMode(){
        byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(EnumMDBCommands.MDB_REQUEST.ENTER_FACTORY_MODE);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send enter factory succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
//            handler.obtainMessage(MDBUtils.BLACK_LOG, "send enter factory succeed !").sendToTarget();
            mdbValues.setFactoryMode(true);
            View view = selectedFragment.getView();
            if(view != null){
                Button button = view.findViewById(R.id.btn_factory_mode);
                button.setText("quit factory mode");
            }
        }
        SystemClock.sleep(200);
    }
    private void sendQuitFactoryMode(){
        byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(EnumMDBCommands.MDB_REQUEST.QUIT_FACTORY_MODE);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send quit factory succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
//            handler.obtainMessage(MDBUtils.BLACK_LOG, "send quit factory succeed !").sendToTarget();
            mdbValues.setFactoryMode(false);
            View view = selectedFragment.getView();
            if(view != null){
                Button button = view.findViewById(R.id.btn_factory_mode);
                button.setText("enter factory mode");
            }
        }
        SystemClock.sleep(200);
    }

    private void sendHardwareDiagnoseTest(){
        byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(EnumMDBCommands.MDB_REQUEST.HARDWARE_DIAGNOSE_TEST);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send hardware diagnose test succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "send hardware diagnose test succeed !").sendToTarget();
        }
        SystemClock.sleep(200);
    }

    //09 03 00 92 6E 0D resp: 09 06 01 92 00 00 0E 5F 0D
    //if version number is 20, resp: 09 06 01 92 00 00 14 59 0D
    private void sendVersionRequest() {
        byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(EnumMDBCommands.MDB_REQUEST.VERSION_REQUEST);
        int write = SerialPortInterface.write(wbytes, 0, wbytes.length);
        handler.obtainMessage(MDBUtils.BLACK_LOG, "Getting version, please wait... ").sendToTarget();
        if (write >= 0) {
            Log.d(TAG, "send get version request succeed !" + ByteConvertStringUtil.buf2StringCompact(wbytes));
        }
        SystemClock.sleep(200);
    }

    private void sendHardwareVersionRequest(){//09 03 00 93 6D 0D resp: 09 05 01 93 00 04 68 0D
        byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(EnumMDBCommands.MDB_REQUEST.HARDWARE_VERSION_REQUEST);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send get hardware version succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
        }
    }

    private void printLogWithBlueRead(String blue, byte[] black){
        handler.obtainMessage(MDBUtils.BLUE_LOG, blue).sendToTarget();
        handler.obtainMessage(MDBUtils.BLACK_LOG, ByteConvertStringUtil.buf2StringCompact(black)).sendToTarget();
    }

    private void printHardwareVersion(byte[] readBytes){
//        String version = sp.substring(9, 11);
//        int hardwareVersionNo = Integer.parseInt(version, 16);
        int hardwareVersionNo = readBytes[3];
        handler.obtainMessage(MDBUtils.BLACK_LOG, "hardware version: " + hardwareVersionNo).sendToTarget();
    }

    private void testMdbMasterSend(){
        new Thread(new Runnable(){
            @Override
            public void run() {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "Init, sending reset request").sendToTarget();
                Log.d(TAG, "Init, sending reset request");
                boolean flagEnd = true;
//                int write = SerialPortInterface.write(writeData, 0, writeData.length);
                boolean isPolled = false;
                byte[] resetRequest = createBytes();
                if(writeData(resetRequest)) {
                    boolean isReadReset = false;
                    boolean isReadSetupPrice = false;
                    boolean isReadEnableReader = false;
                    boolean isReadBeginSession = false;
                    boolean isReadVendDispatch = false;
                    boolean isReadConfig = false;
                    boolean isReadExpansionId = false;
                    boolean isReadVendApproved = false;
                    boolean isReadEnd = false;
                    boolean isWriteReset = false;
                    boolean isWriteSetupPrice = false;
                    boolean isWriteEnableReader = false;
                    boolean isWriteBeginSession = false;
                    boolean isWriteVendDispatch = false;
                    boolean isWriteConfig = false;
                    boolean isWriteExpansionId = false;
                    boolean isWriteVendApproved = false;
                    boolean isWriteEnd = false;
                    while (flagEnd) {
                        isPolled = false;
                        byte[] readResult = new byte[1];
                        int read = SerialPortInterface.read(readResult, readResult.length, 1000);
                        String sp = "";
                        if (read >= 0) {
                            if (ByteConvertStringUtil.buf2StringCompact(readResult).equals("09")) {
                                read = SerialPortInterface.read(readResult, readResult.length, 500);
                                if (read > 0) {
                                    String lenthHex = ByteConvertStringUtil.buf2StringCompact(readResult);
                                    BigInteger item = new BigInteger(lenthHex, 16);
                                    readResult = new byte[item.intValue() + 1];
                                    read = SerialPortInterface.read(readResult, readResult.length, 500);
                                    String stringCompact = ByteConvertStringUtil.buf2StringCompact(readResult);
                                    Log.d(TAG, "read success !!! read result : " + stringCompact);
                                    sp = stringCompact;
                                }
                            } else {
                                sp = "";
                                flagEnd = false;
                            }
                            if (sp.startsWith("01 00")) {
                                if(sp.equals("01 00 00 FF 0D")){
                                    isReadReset = true;
                                }
                                if (!isReadConfig && isReadReset) {
                                    //0011000200000114
                                    byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x11, 0x00, 0x02, 0x00, 0x00, 0x01, 0x14});
                                    if(isWriteConfig){
                                        writeData(createPollBytes());
                                    }else if (writeData(wbytes)) {
                                        handler.obtainMessage(MDBUtils.BLACK_LOG, "sending setup config request").sendToTarget();
                                        isWriteConfig = true;
                                    }
                                }else if (!isReadExpansionId && isReadSetupPrice) {
                                    //0017004944533030303031303030363630324944534d4442564d43434f4e303221
                                    byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x17, 0x00, 0x49, 0x44, 0x53, 0x30, 0x30, 0x30, 0x30, 0x31, 0x30, 0x30, 0x30, 0x36, 0x36, 0x30,
                                            0x32, 0x49, 0x44, 0x53, 0x4D, 0x44, 0x42, 0x56, 0x4D, 0x43, 0x43, 0x4F, 0x4E, 0x30, 0x32, 0x21});
                                    if(isWriteExpansionId){
                                        writeData(createPollBytes());
                                    }else if (writeData(wbytes)) {
                                        handler.obtainMessage(MDBUtils.BLACK_LOG, "sending Expansion ID request").sendToTarget();
                                        isWriteExpansionId = true;
                                    }
                                }else if (!isReadBeginSession && isReadEnableReader) {
                                    //00030064ffffffff00000063
//                                    byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x03, 0x00, 0x64, (byte) 0xFF, (byte) 0xFF, (byte) 0xFF,
//                                            (byte) 0xFF, 0x00, 0x00, 0x00, 0x63});
//                                    if(isWriteBeginSession){
//                                        writeData(createPollBytes());
//                                    }else if (writeData(wbytes)) {
//                                        handler.obtainMessage(MDBUtils.BLACK_LOG, "sending begin session with balance 0064 request").sendToTarget();
                                    isReadBeginSession = true;
                                    isWriteBeginSession = true;
//                                    }
                                }else if (!isReadVendApproved && isReadBeginSession) {
                                    //001300006400e158
                                    byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x13, 0x00, 0x00, 0x64, 0x00, (byte) 0xE1, 0x58});
                                    if(isWriteVendApproved){
                                        writeData(createPollBytes());
                                    }else if (writeData(wbytes)) {
                                        handler.obtainMessage(MDBUtils.BLACK_LOG, "sending vend request with amount 0064").sendToTarget();
                                        isWriteVendApproved = true;
                                    }
                                }else if (!isReadEnd && isReadVendDispatch) {
                                    //00130417
                                    byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x13, 0x04, 0x17});
                                    if(isWriteEnd){
                                        writeData(createPollBytes());
                                    }else if (writeData(wbytes)) {
                                        handler.obtainMessage(MDBUtils.BLACK_LOG, "sending vend session end request").sendToTarget();
                                        isReadEnd = true;
                                        isWriteEnd = true;
                                    }
                                }
                            }
                            if (sp.startsWith("01 01") && !isReadSetupPrice) {
                                isReadConfig = true;
                                //0011012b5c000aa3
                                byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x11, 0x01, 0x2B, 0x5C, 0x00, 0x0A, (byte) 0xA3});
                                if(isWriteSetupPrice){
                                    writeData(createPollBytes());
                                }else if (writeData(wbytes)) {
                                    handler.obtainMessage(MDBUtils.BLACK_LOG, "sending setup max/mix price request").sendToTarget();
                                    isReadSetupPrice = true;
                                    isWriteSetupPrice = true;
                                }
                            }
                            if (sp.startsWith("01 09") && !isReadEnableReader) {
                                isReadExpansionId = true;
                                //00140115
                                byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x14, 0x01, 0x15});
                                if(isWriteEnableReader){
                                    writeData(createPollBytes());
                                }else if (writeData(wbytes)) {
                                    handler.obtainMessage(MDBUtils.BLACK_LOG, "sending enable Reader request").sendToTarget();
                                    isReadEnableReader = true;
                                    isWriteEnableReader = true;
                                }
                            }
                            if (sp.startsWith("01 05") && !isReadVendDispatch) {
                                isReadVendApproved = true;
                                //00130200e1f6
                                byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x13, 0x02, 0x00, (byte) 0xE1, (byte) 0xF6});
                                if(isWriteVendDispatch){
                                    writeData(createPollBytes());
                                }else if (writeData(wbytes)) {
                                    handler.obtainMessage(MDBUtils.BLACK_LOG, "sending vend dispatch request with item 00e1").sendToTarget();
                                    isReadVendDispatch = true;
                                    isWriteVendDispatch = true;
                                }
                            }
                            if (sp.startsWith("01 06")) {
                                byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x13, 0x04, 0x17});
                                if(isWriteEnd){
                                    writeData(createPollBytes());
                                }else if (writeData(wbytes)) {
                                    handler.obtainMessage(MDBUtils.BLACK_LOG, "vend denied!! sending vend session end request").sendToTarget();
                                    isReadEnd = true;
                                    isWriteEnd = true;
                                }
                            }
                            if (sp.startsWith("01 07")) {
                                flagEnd = false;
                            }
                        }else if(!isPolled){
                            if(writeData(createPollBytes())){
                                isPolled = true;
                            }
                        } else if (read == -262254) {
                            handler.obtainMessage(MDBUtils.RED_LOG, "serial port read timeout").sendToTarget();
                        } else {
                            handler.obtainMessage(MDBUtils.RED_LOG, "serial port read failed = " + read).sendToTarget();
                            Log.d(TAG, "read fail!!");
                        }
                    }
                }
                else {
                    Log.e(TAG, "write failed!! write data = " + ByteConvertStringUtil.buf2StringCompact(resetRequest));
                }
            }
        }).start();
    }

    /**
     * 1. test mib
     * send command to serial "Hardware diagnostic test":
     * Eg：
     * Send Data:090300946c0d
     * Recv Data: 09040194006b0d
     * Ps： ifCMDID（0x94）后面的ACK 不为0x00 ，说明MDB 通讯timeout
     * 2.get MIB version
     * send get version cmd
     * Send Data: 090300926e0d
     * Recv Data: 090601920000 0d 600d
     */

    void testMdb() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                open();
                byte[] bytes = {0x09, 0x03, 0x00, (byte) 0x94, (byte) 0x6C, (byte) 0x0D};
                int write = SerialPortInterface.write(bytes, 0, bytes.length);
                if (write >= 0) {
                    handler.obtainMessage(MDBUtils.BLACK_LOG, "testing").sendToTarget();
                    byte[] resulta = new byte[16];
                    int read = SerialPortInterface.read(resulta, resulta.length, 3000);
                    if (read >= 0) {
                        String bytes2Str = ByteConvertStringUtil.buf2StringCompact(subByteArray(resulta, read));
                        if (bytes2Str.startsWith("[09 04 01 94 00")) {
                            handler.obtainMessage(MDBUtils.BLACK_LOG, "read ok: " + bytes2Str).sendToTarget();
                            handler.obtainMessage(MDBUtils.GREEN_LOG, "test ok").sendToTarget();
                        } else {
                            handler.obtainMessage(MDBUtils.RED_LOG, "MDB read timeout! " + bytes2Str).sendToTarget();
                            handler.obtainMessage(MDBUtils.RED_LOG, "test failed!").sendToTarget();
                        }
                    } else if (read == -262254) {
                        handler.obtainMessage(MDBUtils.RED_LOG, "serialport read timeout").sendToTarget();
                    } else {
                        handler.obtainMessage(MDBUtils.RED_LOG, "serialport read failed").sendToTarget();
                    }
                }
                close();
            }
        }).start();
    }

    //e.g. version 0019: sp == 01 92 00 00 13 5A, 00 13 decides 19
    public void printVersionName(byte[] readBytes){
        int high = readBytes[3];
        int low = readBytes[4];
        int version = ((high & 0xFF) << 8) | (low & 0xFF);
        mdbValues.setCurrentVersion(version);
        handler.obtainMessage(MDBUtils.GREEN_LOG, "version = " + version).sendToTarget();
    }

    //可以添加参数 byte[] contentBytes
    private byte[] createBytes(){
        int lrc;
        byte[] contentBytes = {0x00, 0x10, 0x10};
        lrc = LRCCheckInt(contentBytes);
        byte[] dataBytes = {0x09, 0x04, 0x00, 0x10, 0x10, (byte) lrc, (byte) 0x0D};
        return dataBytes;
    }

    private byte[] createPollBytes(){
        int lrc;
        byte[] contentBytes = {0x00, 0x12, 0x12};
        lrc = LRCCheckInt(contentBytes);
        byte[] dataBytes = {0x09, 0x04, 0x00, 0x12, 0x12, (byte) lrc, (byte) 0x0D};
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
        return dataBytes;
    }

    private boolean writeData(byte[] data){
        boolean ret = false;
        int write = SerialPortInterface.write(data, 0, data.length);
        if(Arrays.equals(data, createPollBytes())){
            Log.d(TAG, "polling ...");
        } else if(write >= 0){
            Log.d(TAG, "write data success! write length : " + data.length + "data : " + ByteConvertStringUtil.buf2StringCompact(data));
            ret = true;
        }else{
            Log.e(TAG, "write data failed!! Data : " + ByteConvertStringUtil.buf2StringCompact(data));
        }
        return ret;
    }


    Thread downloadTh;

    byte[] readSDfile(String sdcard_mib_path) {
        File file = new File(sdcard_mib_path);
        try {
            FileInputStream fis = new FileInputStream(file);
            int ch = 0;
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            while ((ch = fis.read()) != -1) {
                baos.write(ch);
            }
            baos.flush();
            baos.close();
            fis.close();
            return baos.toByteArray();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    private void updateFw() {
        if (downloadTh != null && downloadTh.getState() != Thread.State.TERMINATED) {
            Log.d(TAG, "downloadFw  return" + " downloadFw  return");
            handler.obtainMessage(MDBUtils.BLACK_LOG, "downloadFw  return").sendToTarget();
            return;
        }
        listAssets();
        downloadTh = new Thread(new Runnable() {
            @Override
            public void run() {
                boolean downloadSucceed = false;
                SerialPortInterface.close();
                String binPath = Environment.getExternalStorageDirectory().getPath() + "/mib_test.bin";
                byte[] fwTotal = readSDfile(binPath);
                if (fwTotal != null && fwTotal.length > 100) {
                    handler.obtainMessage(MDBUtils.BLACK_LOG, "\nfound sdcard mib firmware ...,\n, " + binPath).sendToTarget();
                } else {
                    try {
//                          binPath = assetFiles.get(assetFiles.size() - 1);
                        InputStream fis = getBinStream();
                        if(fis != null) {
                            fwTotal = toByteArray(fis);
                            fwTotal = Arrays.copyOfRange(fwTotal, 0x80, fwTotal.length);
                            float length = fwTotal.length;
                            handler.obtainMessage(MDBUtils.BLACK_LOG, "\ndownloading mib firmware...,\n " +
                                    "name= " + mdbValues.getFirmwareName() + ", size= " + length +
                                    "\nplease wait...").sendToTarget();
                            byte[][] bytes1 = MDBUtils.splitBytes(fwTotal, 0x100);
                            int count = 0;
                            int open = Stm32ispInterface.ispOpen();
                            if (open >= 0) {
                                handler.obtainMessage(MDBUtils.BLACK_LOG, "stm32 open success").sendToTarget();
                                for (byte[] b : bytes1) {
                                    Stm32ispInterface.ispDownload(b, b.length);
                                    /* progress bar
                                    float writeLength = count += b.length;
                                    float f = writeLength / length;
                                    handler.obtainMessage(3, f * 100).sendToTarget();
                                    */
                                }
                                handler.obtainMessage(MDBUtils.GREEN_LOG, "\ndownload complete!").sendToTarget();
                                downloadSucceed = true;
                                handler.obtainMessage(MDBUtils.ENABLE_ALL_UI).sendToTarget();
                            } else {
                                handler.obtainMessage(MDBUtils.RED_LOG, "stm32 open failed").sendToTarget();
                                downloadSucceed = false;
                            }
                            int i = Stm32ispInterface.ispClose();
                            if (i >= 0) {
                                handler.obtainMessage(MDBUtils.BLACK_LOG, "stm32 close success").sendToTarget();
                            } else {
                                handler.obtainMessage(MDBUtils.RED_LOG, "stm32 close failed").sendToTarget();
                            }
                        } else {
                            downloadSucceed = false;
                            handler.obtainMessage(MDBUtils.RED_LOG, "download firmware failed").sendToTarget();
                            handler.obtainMessage(MDBUtils.ENABLE_ALL_UI).sendToTarget();
                        }
                    } catch (IOException e) {
                        throw new RuntimeException(e);
                    }
                    SystemClock.sleep(100);
                    int result = SerialPortInterface.open("SERIAL_EXT");
                    if(downloadSucceed) {
                        sendVersionRequest();
                    }
                }
            }
        });
        downloadTh.start();
    }

    ArrayList<String> assetFiles = new ArrayList<>();

    private void listAssets() {
        String[] list = new String[0];
        try {
            list = context.getAssets().list("");
        } catch (IOException e) {
            e.printStackTrace();
        }
        for (String fileName : list) {
            if (fileName.endsWith(".bin")) {//fileName.startsWith("mib") &&
//                Log.d(TAG, "assetFile=" + fileName);
                assetFiles.add(fileName);
            }
        }
    }

    private InputStream getBinStream() throws IOException {
        listAssets();
        String file_name = "";
        InputStream fis = null;
        switch (mdbValues.getFirmwareName()){
            case "0014":
                file_name = "WhiteDemon_0014.bin";
                fis = context.getAssets().open(file_name);
                break;
            case "0015":
                file_name = "WhiteDemon_0015.bin";
                fis = context.getAssets().open(file_name);
                break;
            case "0017":
                file_name = "WhiteDemon_0017.bin";
                fis = context.getAssets().open(file_name);
                break;
            case "0018":
                file_name = "WhiteDemon_0018.bin";
                fis = context.getAssets().open(file_name);
                break;
            case "0018_debug":
                file_name = "WhiteDemon_debug_0018.bin";
                fis = context.getAssets().open(file_name);
                break;
            case "0019":
                file_name = "WhiteDemon_0019.bin";
                fis = context.getAssets().open(file_name);
                break;
            case "0020":
                file_name = "WhiteDemon_0020.bin";
                fis = context.getAssets().open(file_name);
                break;
            case "0021":
                file_name = "WhiteDemon_0021.bin";
                fis = context.getAssets().open(file_name);
                break;
            case "0022":
                file_name = "WhiteDemon_0022.bin";
                fis = context.getAssets().open(file_name);
                break;
            default:
                if(Integer.parseInt(mdbValues.getFirmwareName()) >= 23){
                    File filesDir = context.getFilesDir();
                    String filePath = context.getFilesDir().getPath();
                    if(filesDir.exists() && filesDir.isDirectory()){
                        File[] files = filesDir.listFiles();
                        if(files != null && files.length > 0){
                            for(File file : files){
                                if(file.getName().substring(20, 24).equals(mdbValues.getFirmwareName())){
                                    File inputFile = new File(filePath, file.getName());
                                    fis = new FileInputStream(inputFile);
                                }
                            }
                        }
                    }
                } else {
                    fis = null;
                }
                break;
        }
        return fis;
    }

    private int setPulse(){
        int ret = -1;
        if(pulseValues.getPulseDuration() < 0){
            return ret;
        }
        if(pulseValues.getPulseVoltage() != 0 && pulseValues.getPulseVoltage() != 1){
            return ret;
        }
        if(pulseValues.getPulseInterval() <= 0){
            return ret;
        }
        if(pulseValues.getPulseFrequency() <= 0){
            return ret;
        }
        sendSetPulseDuration(pulseValues.getPulseDuration());
        sendSetPulseVoltage(pulseValues.getPulseVoltage());
        return 1;
    }

    public byte[] toByteArray(InputStream fis) throws IOException {
        int ch = 0;
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        while ((ch = fis.read()) != -1) {
            baos.write(ch);
        }
        baos.flush();
        baos.close();
        fis.close();
        return baos.toByteArray();
    }

    void open() {
        int result = SerialPortInterface.open("SERIAL_EXT");
        if (result < 0) {
            onLogSent('e', TAG,"seriaport open failed!" + result);
        } else {
            result = SerialPortInterface.setBaudrate(115200);
            SystemClock.sleep(100);
            onLogSent('d', TAG, "seriaport open success!");
//            if(isSlave)
            testMdbSlaveRead();
        }
    }
    void close() {
        int result = SerialPortInterface.close();
        if (result < 0) {
            onLogSent('e', TAG, "SerialPort close failed : " + result);
        } else {
            onLogSent('d', TAG, "seriaport close success");
        }
    }

    public void test(){

    }

    public static byte[] subByteArray(byte[] byteArray, int length) {
        byte[] arrySub = new byte[length];
        if (length >= 0) System.arraycopy(byteArray, 0, arrySub, 0, length);
        return arrySub;
    }

    public static byte[] subByteArrayIgnore(byte[] byteArray, int startIndex, int length) {
        int realLength = length - startIndex;
        byte[] arrySub = new byte[realLength];
        if (length >= 0) System.arraycopy(byteArray, startIndex, arrySub, 0, realLength);
        return arrySub;
    }

    private byte[] createBytes(String edit) {
        ArrayList<Byte> as = new ArrayList<>();

        int s1 = 9;
        int e1 = 13;

        int len;
        int lrc;
        String[] split = edit.replace(",", " ").split(" ");
        len = split.length + 1;
        for (String spli : split) {
            as.add((byte) Integer.parseInt(spli, 16));
        }
        byte[] by = new byte[as.size()];
        for (int i = 0; i < as.size(); i++) {
            by[i] = as.get(i);
        }
        lrc = LRCCheckInt(by);
        ArrayList<Byte> as2 = new ArrayList<>();

        as2.add((byte) s1);
        as2.add((byte) len);

        for (byte b : by) {
            as2.add(b);
        }

        as2.add((byte) lrc);
        as2.add((byte) e1);

        byte[] by2 = new byte[as2.size()];
        for (int i = 0; i < as2.size(); i++) {
//            Log.d("ByteConvertStringUtil.buf2StringCompact by2","as2.get(i)" +as2.get(i));
            by2[i] = as2.get(i);
        }

        String s = ByteConvertStringUtil.buf2StringCompact(by2);
        Log.d(TAG, "ss = " + s);
        return by2;
    }

    private int LRCCheckInt(byte[] bytes) {
        int l1 = 0;
        for (byte b : bytes) {
            l1 = l1 + (int) b;
        }
        return ~l1 + 1;
    }

    private String LRCCheck(byte[] bytes) {
        int l1 = 0;
        for (byte b : bytes) {
            l1 = l1 + (int) b;
        }
        return Integer.toHexString(~l1 + 1);
    }

    public static String getTimeWithFormat() {
        long time = System.currentTimeMillis();
        SimpleDateFormat sdr = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.CHINA);
        return sdr.format(time);
    }

    @Override
    public void onBackPressed() {
        if(drawerLayout.isDrawerOpen(GravityCompat.START)){
            drawerLayout.closeDrawer(GravityCompat.START);
        }else {
            super.onBackPressed();
        }
    }

    @Override
    public void onLogSent(char c, String tag, String log) {
        switch (c){
            case 'd':
                handler.obtainMessage(MDBUtils.BLACK_LOG, log).sendToTarget();
                Log.d(tag, log);
                break;
            case 'i':
                handler.obtainMessage(MDBUtils.BLACK_LOG, log).sendToTarget();
                Log.i(tag, log);
                break;
            case 'e':
                handler.obtainMessage(MDBUtils.RED_LOG, log).sendToTarget();
                Log.e(tag, log);
                break;
        }
    }

    @Override
    public void onIntValueSent(int type, int value) {
        switch(type){
            case MDBUtils.TYPE_PULSE_INTERVAL:
                pulseValues.setPulseInterval(value);
                break;
            case MDBUtils.TYPE_PULSE_FREQUENCY:
                pulseValues.setPulseFrequency(value);
                break;
            case MDBUtils.TYPE_PULSE_DURATION:
                pulseValues.setPulseDuration(value);
                break;
            case MDBUtils.TYPE_PULSE_VOLTAGE:
                pulseValues.setPulseVoltage(value);
                break;
            case MDBUtils.TYPE_BALANCE:
                mdbValues.setBalance(value);
                break;
            case MDBUtils.TYPE_SPN_MDB_LEVEL:
                mdbValues.setMdbLevel(value);
                break;
            case MDBUtils.ENABLE_ALL_UI:
                handler.obtainMessage(MDBUtils.ENABLE_ALL_UI).sendToTarget();
                break;
            case MDBUtils.DISABLE_ALL_UI:
                handler.obtainMessage(MDBUtils.DISABLE_ALL_UI).sendToTarget();
                break;
            default:
                break;
        }
    }

    @Override
    public void onStringValueSent(int type, String value) {
        switch (type){
            case MDBUtils.TYPE_SPN_FIRMWARE_ITEM:
                mdbValues.setFirmwareName(value);
                break;
            default:
                break;
        }
    }

    @Override
    public void onBooleanValueSent(int type, boolean value) {
        switch (type){
            case MDBUtils.TYPE_CB_CHECK_ALWAYS_IDLE:
                mdbValues.getOptionalFeature().setAlwaysIdleMdb(value);
                break;
            case MDBUtils.TYPE_CB_CHECK_32BIT_MONETARY:
                mdbValues.getOptionalFeature().setMonetaryFormat32Mdb(value);
                break;
            case MDBUtils.TYPE_CB_CHECK_NEGATIVE_VEND:
                mdbValues.getOptionalFeature().setNegativeVendAllowedMdb(value);
                break;
            default:
                break;
        }
    }

    @Override
    public void onOpenClicked() {
        open();
    }

    @Override
    public void onCloseClicked() {
        close();
    }

    @Override
    public void onGetVersionClicked() {
        sendVersionRequest();
    }

    @Override
    public void onUpdateFwClicked() {
        updateFw();
    }

    @Override
    public void onTestClicked() {
        test();
    }

    Handler delayHandler = new Handler();

    @Override
    public void onTriggerPulseClicked() {
        if(pulseValues.getPulseFrequency() < 0){
            return;
        }
        pulseValues.setCurrentSend(pulseValues.getPulseFrequency());
        if(pulseValues.getPulseInterval() < 0){
            return;
        }
        handler.obtainMessage(MDBUtils.DISABLE_ALL_UI).sendToTarget();
        sendVersionRequest();
        if(setPulse() < 0){
            Log.e(TAG, "pulse parameter error!");
            handler.obtainMessage(MDBUtils.BLACK_LOG, "pulse parameter error!").sendToTarget();
            return;
        }
        SystemClock.sleep(500);
        if(mdbValues.getCurrentVersion() >= 19){
            if(!TRIGGER_PULSE_ONE_SHOT) {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse processing, please wait ...").sendToTarget();
                sendTriggerPulse();
                for (int i = 1; i < pulseValues.getPulseFrequency(); ++i) {
                    int delayTime = i * (pulseValues.getPulseInterval() + pulseValues.getPulseDuration());
                    delayHandler.postDelayed(new Runnable() {
                        @Override
                        public void run() {
                            sendTriggerPulse();
                        }
                    }, delayTime);
                }
            } else {
                if(mdbValues.getCurrentVersion() >= 22) {
                    handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse processing, please wait ...").sendToTarget();
                    sendTriggerPulseOneShot();
                    delayHandler.postDelayed(new Runnable() {
                        @Override
                        public void run() {
                            handler.obtainMessage(MDBUtils.ENABLE_ALL_UI).sendToTarget();
                        }
                    }, pulseValues.getPulseFrequency() * (pulseValues.getPulseInterval() + pulseValues.getPulseDuration()));
                }
                else {
                    handler.obtainMessage(MDBUtils.RED_LOG, "version is not equal or greater than 22, " +
                            "trigger pulse failed, please update FW version").sendToTarget();
                }
            }
        }else{
            handler.obtainMessage(MDBUtils.RED_LOG, "version is not equal or greater than 19, trigger pulse failed, " +
                    "please update FW version").sendToTarget();
        }
    }

    @Override
    public void onSetPulseClicked() {
        if(setPulse() < 0){
            Log.e(TAG, "pulse parameter error!");
            handler.obtainMessage(MDBUtils.BLACK_LOG, "pulse parameter error!").sendToTarget();
        }
    }

    @Override
    public void onFactoryModeClicked() {
        View view = selectedFragment.getView();
        if(view != null){
            Button button = view.findViewById(R.id.btn_factory_mode);
            String buttonText = button.getText().toString();
            if(buttonText.equalsIgnoreCase("enter factory mode")){
                sendEnterFactoryMode();
            } else if (buttonText.equalsIgnoreCase("quit factory mode")) {
                sendQuitFactoryMode();
            }
        }
    }

    @Override
    public void onGetHardwareVersionClicked() {
        sendHardwareVersionRequest();
    }

    @Override
    public void onDiagnoseHardwareClicked() {
        sendHardwareDiagnoseTest();
    }

    @Override
    public void onTestVmcClicked() {
        testVmc();
    }

    public void printLogAndText(char c, String s){
        onLogSent(c, TAG, s);
    }

    private void disableAllUi(){
        NavigationView navigationView = findViewById(R.id.nav_view);
        MenuItem menuItem = navigationView.getCheckedItem();
        if(menuItem != null){
            int itemId = menuItem.getItemId();
            if(itemId == R.id.nav_home){
                disableHomeUi();
            } else if(itemId == R.id.nav_pulse){
                disablePulseUi();
            } else if(itemId == R.id.nav_transaction){
                disableTransactionUi();
            }
        }
    }

    private void enableAllUi(){
        NavigationView navigationView = findViewById(R.id.nav_view);
        MenuItem menuItem = navigationView.getCheckedItem();
        if(menuItem != null){
            int itemId = menuItem.getItemId();
            if(itemId == R.id.nav_home){
                enableHomeUi();
            } else if(itemId == R.id.nav_pulse){
                enablePulseUi();
            } else if(itemId == R.id.nav_transaction){
                enableTransactionUi();
            }
        }
    }

    private void disablePulseUi(){
        EditText etDuration = selectedFragment.getView().findViewById(R.id.et_duration);
        etDuration.setEnabled(false);
        EditText etFrequency = selectedFragment.getView().findViewById(R.id.et_frequency);
        etFrequency.setEnabled(false);
        EditText etLatency = selectedFragment.getView().findViewById(R.id.et_latency);
        etLatency.setEnabled(false);

        Button btnTriggerPulse = selectedFragment.getView().findViewById(R.id.btn_trigger_pulse);
        btnTriggerPulse.setEnabled(false);
        Button btnSetPulse = selectedFragment.getView().findViewById(R.id.btn_set_pulse);
        btnSetPulse.setEnabled(false);

        Spinner spnVoltage = selectedFragment.getView().findViewById(R.id.spn_voltage);
        spnVoltage.setEnabled(false);
    }

    private void enablePulseUi(){
        EditText etDuration = selectedFragment.getView().findViewById(R.id.et_duration);
        etDuration.setEnabled(true);
        EditText etFrequency = selectedFragment.getView().findViewById(R.id.et_frequency);
        etFrequency.setEnabled(true);
        EditText etLatency = selectedFragment.getView().findViewById(R.id.et_latency);
        etLatency.setEnabled(true);

        Button btnTriggerPulse = selectedFragment.getView().findViewById(R.id.btn_trigger_pulse);
        btnTriggerPulse.setEnabled(true);
        Button btnSetPulse = selectedFragment.getView().findViewById(R.id.btn_set_pulse);
        btnSetPulse.setEnabled(true);

        Spinner spnVoltage = selectedFragment.getView().findViewById(R.id.spn_voltage);
        spnVoltage.setEnabled(true);
    }

    private void disableTransactionUi(){
        Button btnSetBalance = selectedFragment.getView().findViewById(R.id.btn_set_balance);
        btnSetBalance.setEnabled(false);
        EditText etBalance = selectedFragment.getView().findViewById(R.id.et_balance);
        etBalance.setEnabled(false);
        Spinner spnMdbLevel = selectedFragment.getView().findViewById(R.id.spn_mdb_level);
        spnMdbLevel.setEnabled(false);
    }

    private void enableTransactionUi(){
        Button btnSetBalance = selectedFragment.getView().findViewById(R.id.btn_set_balance);
        btnSetBalance.setEnabled(true);
        EditText etBalance = selectedFragment.getView().findViewById(R.id.et_balance);
        etBalance.setEnabled(true);
        Spinner spnMdbLevel = selectedFragment.getView().findViewById(R.id.spn_mdb_level);
        spnMdbLevel.setEnabled(true);
    }

    private void disableHomeUi(){
        Button btnGetVersion = selectedFragment.getView().findViewById(R.id.btn_get_version);
        btnGetVersion.setEnabled(false);
        Button btnUpdateFw = selectedFragment.getView().findViewById(R.id.btn_update_fw);
        btnUpdateFw.setEnabled(false);
        Button btnFactoryMode = selectedFragment.getView().findViewById(R.id.btn_factory_mode);
        btnFactoryMode.setEnabled(false);
        Button btnGetHardwareVersion = selectedFragment.getView().findViewById(R.id.btn_get_hardware_version);
        btnGetHardwareVersion.setEnabled(false);
        Spinner spnWhiteDemon = selectedFragment.getView().findViewById(R.id.spn_white_demon);
        spnWhiteDemon.setEnabled(false);
    }

    private void enableHomeUi(){
        Button btnGetVersion = selectedFragment.getView().findViewById(R.id.btn_get_version);
        btnGetVersion.setEnabled(true);
        Button btnUpdateFw = selectedFragment.getView().findViewById(R.id.btn_update_fw);
        btnUpdateFw.setEnabled(true);
        Button btnFactoryMode = selectedFragment.getView().findViewById(R.id.btn_factory_mode);
        btnFactoryMode.setEnabled(true);
        Button btnGetHardwareVersion = selectedFragment.getView().findViewById(R.id.btn_get_hardware_version);
        btnGetHardwareVersion.setEnabled(true);
        Spinner spnWhiteDemon = selectedFragment.getView().findViewById(R.id.spn_white_demon);
        spnWhiteDemon.setEnabled(true);
    }

    public void testVmc(){
        serverThread.start();
        clientThread.start();
    }

    Thread serverThread = new Thread(new Runnable() {
        @Override
        public void run() {
            try {
                ServerSocket serverSocket = new ServerSocket(12345);
                Socket socket = serverSocket.accept();
                DataInputStream dis = new DataInputStream(socket.getInputStream());
                DataOutputStream dos = new DataOutputStream(socket.getOutputStream());
                Item item = new Item();
                while (true) {
                    byte[] byteArr = new byte[1];
                    // read start flag 0x09, 1 bit
                    int readResult = dis.read(byteArr, 0, byteArr.length);
                    Log.d(TAG, "server read 1 byte success: " + ByteConvertStringUtil.buf2StringCompact(byteArr));
                    if(ByteConvertStringUtil.buf2StringCompact(byteArr).equals("0D")){
                        continue;
                    }
                    if (readResult >= 0) {
                        if (ByteConvertStringUtil.buf2StringCompact(byteArr).equals("09")) {
                            // read data length, 1 bit
                            readResult = dis.read(byteArr, 0, byteArr.length);
                            if (readResult > 0) {
                                Log.d(TAG, "server read length success: " + ByteConvertStringUtil.buf2StringCompact(byteArr));
                                String lenthHex = ByteConvertStringUtil.buf2StringCompact(byteArr);
                                BigInteger length = new BigInteger(lenthHex, 16);
                                // read data
                                byteArr = new byte[length.intValue()];
                                readResult = dis.read(byteArr, 0, byteArr.length);
                                if (readResult > 0) {
                                    Log.i(TAG, "server read success: " + ByteConvertStringUtil.buf2StringCompact(byteArr));
                                } else {
                                    Log.e(TAG, "server read failed");
                                }
                                processData(item, byteArr, dos);
                            }
                            else {
                                Log.e(TAG, "read length failed!");
                            }
                        }
                    }

                }
//                socket.close();
//                serverSocket.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    });

    private void processData(Item item, byte[] readBytes, DataOutputStream dos) throws IOException {
//        Log.i(TAG, "readBytes:  " + ByteConvertStringUtil.buf2StringCompact(readBytes));
        if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x10, 0x10}, 3)) {
            sendReset(dos);
            mdbValues.setOptionalFeature(new OptionalFeature());
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x11, 0x00}, 3)) { //Setup config data
            mdbValues.setVmcLevel(readBytes[3]);
            sendSetupConfig(item, dos);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x11, 0x01}, 3)) { // Setup Max/Min Price
            sendSetupPrice(dos);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x17, 0x00}, 3)) { //Expansion ID
            sendExpansionRequestID(dos);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x17, 0x04}, 3)) { //Expansion optional feature bits
            mdbValues.getOptionalFeature().setOptionalFeatureBitsVmc(readBytes[6]);
//            Log.i(TAG, "readBytes[6]: " + Integer.toHexString(readBytes[6]));
//            Log.i(TAG, "optional feature bits: " + Integer.toHexString(mdbValues.getOptionalFeature().getOptionalFeatureBits()));
//            Log.i(TAG, "optional feature: " + mdbValues.getOptionalFeature().toString());
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x14, 0x00}, 3)
                || compareByteArrayHead(readBytes, new byte[]{0x00, 0x14, 0x01}, 3)) {//Enable Reader
            sendEnableReader(dos);
            startTransaction(dos);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, 0x00}, 2)) {
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Ready! Please input item ID").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x15, 0x00}, 3)){ //Revalue approved
            sendRevalueApproved(dos);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x15, 0x01}, 3)){ //Revalue limited
            sendRevalueLimit(dos);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x13, 0x00}, 3)) { // vend request
            //e.g. 00 13 00 00 A0 00 01 B4 98
            //amount: 00 A0
            //id: 00 01
            if(!mdbValues.getOptionalFeature().isMonetaryFormat32Mdb()) {
                item.setItemAmount(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[3], readBytes[4]}).replace(" ", ""));
                item.setItemId(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[5], readBytes[6]}).replace(" ", ""));
                item.calculatePrice();
                BigInteger itemid = new BigInteger(item.getItemId(), 16);
                item.setItemId(itemid.toString());
            } else {
                item.setItemAmount(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[3], readBytes[4], readBytes[5], readBytes[6]}).replace(" ", ""));
                item.setItemId(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[7], readBytes[8]}).replace(" ", ""));
                item.calculatePrice();
                BigInteger itemid = new BigInteger(item.getItemId(), 16);
                item.setItemId(itemid.toString());
            }
            handler.obtainMessage(MDBUtils.DIALOG_READCARD, item).sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x13, 0x02}, 3)) { //vend dispatch
            sendVendSuccess(readBytes, dos);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x13, 0x04}, 3)) { //vend session end
            sendSessionEnd(dos);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x13, 0x06}, 3)){ //negative vend request
            if( mdbValues.getMdbLevel() == 3
            &&  mdbValues.getVmcLevel() == 3
            &&  mdbValues.getOptionalFeature().isNegativeVendAllowedMdb()
            &&  mdbValues.getOptionalFeature().isNegativeVendAllowedVmc()) {
                Item item1 = new Item();
                if(!mdbValues.getOptionalFeature().isMonetaryFormat32Mdb()) {
                    item1.setItemAmount(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[3], readBytes[4]}).replace(" ", ""));
                    item1.setItemId(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[5], readBytes[6]}).replace(" ", ""));
                    item1.calculatePrice();
                    BigInteger itemid = new BigInteger(item1.getItemId(), 16);
                    item1.setItemId(itemid.toString());
                } else {
                    item1.setItemAmount(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[3], readBytes[4], readBytes[5], readBytes[6]}).replace(" ", ""));
                    item1.setItemId(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[7], readBytes[8]}).replace(" ", ""));
                    item1.calculatePrice();
                    BigInteger itemid = new BigInteger(item1.getItemId(), 16);
                    item1.setItemId(itemid.toString());
                }
                sendNegativeVendApproved(readBytes, item1, dos);
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x14, 0x02}, 3)){ //reader cancel
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Reader cancelled").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x90, 0x03}, 3)) { //set duration
            handler.obtainMessage(MDBUtils.BLACK_LOG, "set pulse duration succeed").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x90, 0x04}, 3)) { //set voltage
            handler.obtainMessage(MDBUtils.BLACK_LOG, "set pulse voltage succeed").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x90, 0x05}, 3)){ //factory mode
            if(mdbValues.isFactoryMode()) {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "enter factory mode succeed").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "quit factory mode succeed").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x92, 0x00}, 3)) { //get version
            handler.obtainMessage(MDBUtils.BLACK_LOG, "get result:" +
                    ByteConvertStringUtil.buf2StringCompact(readBytes)).sendToTarget();
            printVersionName(readBytes);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x94}, 2)) { //diagnose hardware
            if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x94, 0x00}, 3)) {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "hardware diagnose test received").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "hardware diagnose test open failed").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x93}, 2)) { //get hardware version
            if(compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x93, 0x00}, 3)) {
                printHardwareVersion(readBytes);
            } else {
                handler.obtainMessage(MDBUtils.RED_LOG, "get hardware version failed!").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, (byte)0x13, 0x01}, 3)) {  // vend denied
            Log.d(TAG, "00 13 01, vending auto cancel, denied");
            sendVendDedied(dos);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, 0x20, 0x00}, 3)) { //trigger pulse
            //receive: 09 04 01 20 00 DF 0D
            pulseValues.setCurrentSend(pulseValues.getCurrentSend() - 1);
            if (pulseValues.getCurrentSend() == 0) {
                handler.obtainMessage(MDBUtils.ENABLE_ALL_UI).sendToTarget();
                handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse " + pulseValues.getPulseFrequency() + " times succeed").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse " + (pulseValues.getPulseFrequency() - pulseValues.getCurrentSend())).sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x21}, 2)) { //trigger pulse one shot resp
            if(compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x21, 0x00}, 3)) {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse succeed").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.RED_LOG, "trigger pulse failed!").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x0d}, 1)){
            Log.d(TAG, "0D, end of data");
        } else {
            handler.obtainMessage(MDBUtils.BLACK_LOG, "read failed: " +
                    ByteConvertStringUtil.buf2StringCompact(readBytes)).sendToTarget();
        }
    }

    private void sendReset(DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.RESET);
        dos.write(wbytes);
        Log.d(TAG, "sendReset:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }
    //readBytes = "00 11 00 02 00 00 01 14"
    private void sendSetupConfig(Item item, DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.SETUP_CONFIG, mdbValues, item);
        dos.write(wbytes);
        Log.d(TAG, "sendSetupConfig:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }
    private void sendSetupPrice(DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.SETUP_PRICE);
        dos.write(wbytes);
        Log.d(TAG, "sendSetupPrice:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }
    private void sendExpansionRequestID(DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.EXPANSION_REQUESTID, mdbValues);
        dos.write(wbytes);
        Log.d(TAG, "sendExpansionRequestID:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }
    private void sendEnableReader(DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.ENABLE_READER);
        dos.write(wbytes);
        Log.d(TAG, "sendEnableReader:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }
    private void sendRevalueApproved(DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.REVALUE_APPROVED);
        dos.write(wbytes);
        Log.d(TAG, "sendRevalueApproved:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }
    private void sendRevalueLimit(DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.REVALUE_LIMIT_AMOUNT);
        dos.write(wbytes);
        Log.d(TAG, "sendRevalueLimit:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }

    private void sendVendSuccess(byte[] readBytes, DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.VEND_SUCCESS);
        dos.write(wbytes);
        Log.d(TAG, "sendVendSuccess: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
        int itemIdHigh = readBytes[3];
        int itemIdLow = readBytes[4];
        int itemId = ((itemIdHigh & 0xFF) << 8) | (itemIdLow & 0xFF);
        handler.obtainMessage(MDBUtils.DIALOG_ITEM, "Vend success! item:" + itemId).sendToTarget();
    }
    private void sendSessionEnd(DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.END_SESSION);
        dos.write(wbytes);
        Log.d(TAG, "sendSessionEnd: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
        SystemClock.sleep(2000);
        startTransaction();
    }
    private void sendVendApproved(Item item, DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.VEND_APPROVED, item);
        dos.write(wbytes);
        Log.d(TAG, "sendVendApproved: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }
    private void sendVendDedied(DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.VEND_DENIED);
        dos.write(wbytes);
        Log.d(TAG, "sendVendDedied: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }
    private void sendNegativeVendApproved(byte[] readBytes, Item item, DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.NEGATIVE_VEND_APPROVED, mdbValues, item);
        dos.write(wbytes);
        Log.d(TAG, "sendNegativeVendApproved: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }

    //begin session
    private void startTransaction(DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(EnumMDBCommands.MDB_REQUEST.BEGIN_SESSION, mdbValues);
        dos.write(wbytes);
        Log.d(TAG, "startTransaction: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }
    private void sendReaderCancelled(String sp, DataOutputStream dos) throws IOException {
        byte[] wbytes = EnumMDBCommands.getResponseOfMdbCommands(EnumMDBCommands.MDB_RESPONSE.READER_CANCELLED);
        dos.write(wbytes);
        Log.d(TAG, "sendReaderCancelled: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
    }

    Thread clientThread = new Thread(new Runnable() {
        @Override
        public void run() {
            try {
                Socket socket = new Socket("localhost", 12345);
                DataOutputStream dos = new DataOutputStream(socket.getOutputStream());
                EnumMDBCommands.MDB_REQUEST mdb_request = EnumMDBCommands.MDB_REQUEST.NEGATIVE_VEND_APPROVED;
                byte[] wbytes = EnumMDBCommands.getRequestOfMdbCommands(mdb_request);
                for(int i = 0; i < 1; i++) {
                    dos.write(wbytes);
                    Log.d(TAG, "send:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
                    SystemClock.sleep(1000);
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    });
}