package com.wizarpos.mdbtest;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.Spinner;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.wizarpos.util.MDBUtils;

public class TransactionFragment extends Fragment implements View.OnClickListener {

	private Context context;
	private DataSendListener dataSendListener;
	private final String TAG = "TransactionFragment";
	private Button btnSetBalance;
	private EditText etBalance;
	private CheckBox cbAlwaysIdle, cb32BitMonetary, cbNegativeVend;
	private Spinner spnMdbLevel;
	private String spnItemMdbLevel;
	private int selectedSpnPositon = -1;

	@Nullable
	@Override
	public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
		View view = inflater.inflate(R.layout.fragment_transaction, container, false);
		btnSetBalance = view.findViewById(R.id.btn_set_balance);

		btnSetBalance.setOnClickListener(this);
		etBalance = view.findViewById(R.id.et_balance);
		spnMdbLevel = view.findViewById(R.id.spn_mdb_level);
		cbAlwaysIdle = view.findViewById(R.id.cb_always_idle);
		cb32BitMonetary = view.findViewById(R.id.cb_32bit_monetary);
		cbNegativeVend = view.findViewById(R.id.cb_negative_vend);

		cbAlwaysIdle.setChecked(false);
		cb32BitMonetary.setChecked(false);
		cbNegativeVend.setChecked(false);

		context = getContext();

		ArrayAdapter<CharSequence> adapterMdbLevel = ArrayAdapter.createFromResource(context,
				R.array.spn_mdb_level, android.R.layout.simple_spinner_item);
		adapterMdbLevel.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spnMdbLevel.setAdapter(adapterMdbLevel);
		if(savedInstanceState != null){
			selectedSpnPositon = savedInstanceState.getInt("selectedSpnPositon", -1);
		}
		if(selectedSpnPositon != -1){
			spnMdbLevel.setSelection(selectedSpnPositon);
		}
		spnMdbLevel.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			@Override
			public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
				selectedSpnPositon = i;
				spnItemMdbLevel = spnMdbLevel.getSelectedItem().toString();
				int nMdbLevel = Integer.parseInt(spnItemMdbLevel);
				dataSendListener.onIntValueSent(MDBUtils.TYPE_SPN_MDB_LEVEL, nMdbLevel);
			}
			@Override
			public void onNothingSelected(AdapterView<?> adapterView) {

			}

		});
		cbAlwaysIdle.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				dataSendListener.onBooleanValueSent(MDBUtils.TYPE_CB_CHECK_ALWAYS_IDLE, isChecked);
			}
		});
		cb32BitMonetary.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				dataSendListener.onBooleanValueSent(MDBUtils.TYPE_CB_CHECK_32BIT_MONETARY, isChecked);
			}
		});
		cbNegativeVend.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				dataSendListener.onBooleanValueSent(MDBUtils.TYPE_CB_CHECK_NEGATIVE_VEND, isChecked);
			}
		});

		return view;
	}

	@Override
	public void onSaveInstanceState(@NonNull Bundle outState) {
		super.onSaveInstanceState(outState);
		outState.putInt("selectedSpnPositon", selectedSpnPositon);
	}

	@Override
	public void onClick(View v) {
		switch(v.getId()){
			case R.id.btn_set_balance:
				dataSendListener.onIntValueSent(MDBUtils.TYPE_BALANCE, getBalance());
				dataSendListener.onLogSent('d', TAG, "set balance succeed");
//				dataSendListener.onSetBalanceClicked();
				break;
			default:
				break;
		}

	}

	//default: 100
	public int getBalance(){
		int ret = -1;
		String sContent = etBalance.getText().toString();
		if(sContent.trim().isEmpty()){
			dataSendListener.onLogSent('d', TAG, "balance is empty, will be set to default: 100");
			return 100;
		}
		try{
			ret = Integer.parseInt(sContent);
		}catch(Exception e){
			dataSendListener.onLogSent('e', TAG, "balance value must be a number");
			return ret;
		}
		if(ret < 0 || ret > 65535){
			dataSendListener.onLogSent('e', TAG, "error: balance value range: 0~65535");
		}
		return ret;
	}

	@Override
	public void onAttach(Context context) {
		super.onAttach(context);
		try {
			dataSendListener = (DataSendListener) context;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
}
