package com.wizarpos.util;


import android.os.Build;
import android.util.Log;

import com.wizarpos.mdbtest.Item;
import com.wizarpos.values.MDBValues;
import com.wizarpos.values.PulseValues;

import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;


public class EnumMDBCommands {

	public enum MDB_RESPONSE {
		RESET,
		SETUP_CONFIG,
		SETUP_PRICE,
		EXPANSION_REQUESTID,
		ENABLE_READER,
		REVALUE_APPROVED,
		REVALUE_LIMIT_AMOUNT,
		VEND_SUCCESS,
		END_SESSION,
		VEND_APPROVED,
		VEND_DENIED,
		NEGATIVE_VEND_APPROVED,
		READER_CANCELLED,

	}

	public enum MDB_REQUEST {
		VERSION_REQUEST,
		TRIGGER_PULSE,
		TRIGGER_PULSE_ONESHOT,
		SET_PULSE_DURATION,
		SET_PULSE_VOLTAGE,
		ENTER_FACTORY_MODE,
		QUIT_FACTORY_MODE,
		HARDWARE_DIAGNOSE_TEST,
		HARDWARE_VERSION_REQUEST,

		RESET,
		SETUP_CONFIG,
		SETUP_PRICE,
		EXPANSION_REQUEST_ID,
		EXPANSION_ENABLE_OPTIONS,
		ENABLE_READER,
		BEGIN_SESSION,
		VEND_REQUEST,
		VEND_DISPATCH,
		END_SESSION,
		NEGATIVE_VEND_APPROVED,
	}

	static Map<MDB_RESPONSE, Function<Object[], byte[]>> mdbResponseMap = new HashMap<>();
	static Map<MDB_REQUEST, Function<Object[], byte[]>> mdbRequestMap = new HashMap<>();

	public static byte[] getResponseOfMdbCommands(MDB_RESPONSE mdbResponse, Object... args) {
		{
			mdbResponseMap.put(MDB_RESPONSE.RESET, EnumMDBCommands::getResponseReset);
			mdbResponseMap.put(MDB_RESPONSE.SETUP_CONFIG, EnumMDBCommands::getResponseSetupConfig);
			mdbResponseMap.put(MDB_RESPONSE.SETUP_PRICE, EnumMDBCommands::getResponseSetupPrice);
			mdbResponseMap.put(MDB_RESPONSE.EXPANSION_REQUESTID, EnumMDBCommands::getResponseExpansionRequestID);
			mdbResponseMap.put(MDB_RESPONSE.ENABLE_READER, EnumMDBCommands::getResponseEnableReader);
			mdbResponseMap.put(MDB_RESPONSE.REVALUE_APPROVED, EnumMDBCommands::getResponseRevalueApproved);
			mdbResponseMap.put(MDB_RESPONSE.REVALUE_LIMIT_AMOUNT, EnumMDBCommands::getResponseRevalueLimitAmount);
			mdbResponseMap.put(MDB_RESPONSE.VEND_SUCCESS, EnumMDBCommands::getResponseVendSuccess);
			mdbResponseMap.put(MDB_RESPONSE.END_SESSION, EnumMDBCommands::getResponseEndSession);
			mdbResponseMap.put(MDB_RESPONSE.VEND_APPROVED, EnumMDBCommands::getResponseVendApproved);
			mdbResponseMap.put(MDB_RESPONSE.VEND_DENIED, EnumMDBCommands::getResponseVenddenied);
			mdbResponseMap.put(MDB_RESPONSE.NEGATIVE_VEND_APPROVED, EnumMDBCommands::getResponseNegativeVendApproved);

			mdbResponseMap.put(MDB_RESPONSE.READER_CANCELLED, EnumMDBCommands::getResponseReaderCancelled);

		}

		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
			return mdbResponseMap.get(mdbResponse).apply(args);
		} else {
			Log.e("EnumMDBCommands", "Error: SDK version is less than 24");
			return null;
		}
	}

	public static byte[] getRequestOfMdbCommands(MDB_REQUEST mdbRequest, Object... args) {
		{
			mdbRequestMap.put(MDB_REQUEST.VERSION_REQUEST, EnumMDBCommands::getRequestVersionRequest);
			mdbRequestMap.put(MDB_REQUEST.TRIGGER_PULSE, EnumMDBCommands::getRequestTriggerPulse);
			mdbRequestMap.put(MDB_REQUEST.TRIGGER_PULSE_ONESHOT, EnumMDBCommands::getRequestTriggerPulseOneshot);
			mdbRequestMap.put(MDB_REQUEST.SET_PULSE_DURATION, EnumMDBCommands::getRequestSetPulseDuration);
			mdbRequestMap.put(MDB_REQUEST.SET_PULSE_VOLTAGE, EnumMDBCommands::getRequestSetPulseVoltage);
			mdbRequestMap.put(MDB_REQUEST.ENTER_FACTORY_MODE, EnumMDBCommands::getRequestEnterFactoryMode);
			mdbRequestMap.put(MDB_REQUEST.QUIT_FACTORY_MODE, EnumMDBCommands::getRequestQuitFactoryMode);
			mdbRequestMap.put(MDB_REQUEST.HARDWARE_DIAGNOSE_TEST, EnumMDBCommands::getRequestHardwareDiagnoseTest);
			mdbRequestMap.put(MDB_REQUEST.HARDWARE_VERSION_REQUEST, EnumMDBCommands::getRequestHardwareVersionRequest);

			mdbRequestMap.put(MDB_REQUEST.RESET, EnumMDBCommands::getRequestReset);
			mdbRequestMap.put(MDB_REQUEST.SETUP_CONFIG, EnumMDBCommands::getRequestSetupConfig);
			mdbRequestMap.put(MDB_REQUEST.SETUP_PRICE, EnumMDBCommands::getRequestSetupPrice);
			mdbRequestMap.put(MDB_REQUEST.EXPANSION_REQUEST_ID, EnumMDBCommands::getRequestExpansionRequestID);
			mdbRequestMap.put(MDB_REQUEST.EXPANSION_ENABLE_OPTIONS, EnumMDBCommands::getRequestExpansionEnableOptions);
			mdbRequestMap.put(MDB_REQUEST.ENABLE_READER, EnumMDBCommands::getRequestExpansionEnableReader);
			mdbRequestMap.put(MDB_REQUEST.BEGIN_SESSION, EnumMDBCommands::getRequestBeginSession);
			mdbRequestMap.put(MDB_REQUEST.VEND_REQUEST, EnumMDBCommands::getRequestVendRequest);
			mdbRequestMap.put(MDB_REQUEST.VEND_DISPATCH, EnumMDBCommands::getRequestVendDispatch);
			mdbRequestMap.put(MDB_REQUEST.END_SESSION, EnumMDBCommands::getRequestEndSession);
			mdbRequestMap.put(MDB_REQUEST.NEGATIVE_VEND_APPROVED, EnumMDBCommands::getRequestNegativeVendApproved);

		}
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
			return mdbRequestMap.get(mdbRequest).apply(args);
		} else {
			Log.e("EnumMDBCommands", "Error: SDK version is less than 24");
			return null;
		}
	}

	public static <T> byte[] getResponseReset(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_RESET};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseSetupConfig(T... args) {
		MDBValues mdbValues = (MDBValues) args[0];
		Item item = (Item) args[1];

		byte[] dataBytes = {MDBUtils.MDB_CMD_READER_CONFIG};
		byte[] readFeatureLevel = {(byte) mdbValues.getMdbLevel()}; //Reader Feature Level
		byte[] countryCode = {0x11, 0x56};//Country Code
		byte[] scaleFactor = {0x01};//Scale Factor (x when calculating price)
		item.setPriceX(Integer.toHexString(scaleFactor[0]));
		byte[] decimalPlaces = {0x02};//Decimal Places (y when calculating price)
		item.setPriceY(Integer.toHexString(decimalPlaces[0]));
		byte[] applicationMaxRespTime = {0x59};//Application Maximum Response Time
		byte[] miscellanenousOptions = {0x0d};//Miscellaneous Options
		dataBytes = MDBUtils.connectBytes(dataBytes, readFeatureLevel, countryCode, scaleFactor,
				decimalPlaces, applicationMaxRespTime, miscellanenousOptions);
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseSetupPrice(T... args) {
		byte[] dataBytes = new byte[1];
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseExpansionRequestID(T... args) {
		MDBValues mdbValues = (MDBValues) args[0];
		byte[] dataBytes = {MDBUtils.MDB_CMD_EXPANSION_PERIPHERALID};
		byte[] manufacturerCode = {0x77, 0x7a, 0x70};//Manufacturer Code
		byte[] serialNumber = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01};//Serial Number
		byte[] modelNumber = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01};//Model Number
		byte[] softwareVersion = {0x00, 0x01};//Software Version
		dataBytes = MDBUtils.connectBytes(dataBytes, manufacturerCode, serialNumber, modelNumber, softwareVersion);
		if(mdbValues.getMdbLevel() == 3 && mdbValues.getVmcLevel() == 3) {
			byte[] optionalFeatureBits = {0x00, 0x00, 0x00, mdbValues.getOptionalFeature().getOptionalFeatureBitsMdb()};//Optional Feature Bits
			dataBytes = MDBUtils.connectBytes(dataBytes, optionalFeatureBits);
		}
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseEnableReader(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_COMMON_REQUEST};
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseRevalueApproved(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_REVALUE_APPROVED};
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseRevalueLimitAmount(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_REVALUE_LIMIT_AMOUNT};
		byte[] revalueLimitAmount = {0x27, 0x10};//Revalue Limit Amount
		dataBytes = MDBUtils.connectBytes(dataBytes, revalueLimitAmount);
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseVendSuccess(T... args) {
		byte[] dataBytes = new byte[1];
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseEndSession(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_END_SESSION};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseVendApproved(T... args) {
		Item item = (Item) args[0];
		byte[] dataBytes = {MDBUtils.MDB_CMD_VEND_APPROVED};
		byte[] itemId = {(byte) Integer.parseInt(item.getItemAmount().substring(0,2), 16),
				(byte) Integer.parseInt(item.getItemAmount().substring(2), 16)};
		dataBytes = MDBUtils.connectBytes(dataBytes, itemId);
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseVenddenied(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_VEND_DENIED};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseNegativeVendApproved(T... args) {
		MDBValues mdbValues = (MDBValues) args[0];
		Item item = (Item) args[1];
		byte[] dataBytes = {MDBUtils.MDB_CMD_NEGATIVE_VEND_APPROVED};
		byte[] itemAmount;
		if(!mdbValues.getOptionalFeature().isMonetaryFormat32Mdb()){
			itemAmount = new byte[]{(byte) Integer.parseInt(item.getItemAmount().substring(0, 2), 16),
					(byte) Integer.parseInt(item.getItemAmount().substring(2), 16)};
		} else {
			itemAmount = new byte[]{(byte) Integer.parseInt(item.getItemAmount().substring(0, 2), 16),
					(byte) Integer.parseInt(item.getItemAmount().substring(2, 4), 16),
					(byte) Integer.parseInt(item.getItemAmount().substring(4, 6), 16),
					(byte) Integer.parseInt(item.getItemAmount().substring(6), 16)};
		}
		dataBytes = MDBUtils.connectBytes(dataBytes, itemAmount);
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}

	public static <T> byte[] getResponseReaderCancelled(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_READER_CANCEL};
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_RESPONSE, dataBytes);
	}


	public static <T> byte[] getRequestTriggerPulse(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_TRIGGER_PULSE};
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST,dataBytes);
	}

	public static <T> byte[] getRequestTriggerPulseOneshot(T... args) {
		PulseValues pulseValues = (PulseValues) args[0];
		byte[] dataBytes = {MDBUtils.MDB_CMD_TRIGGER_PULSE_ONESHOT};
		byte[] voltageByte = MDBUtils.intToByteArray(pulseValues.getPulseVoltage());
		byte[] durationByte = MDBUtils.intToByteArray(pulseValues.getPulseDuration());
		byte[] intervalByte = MDBUtils.intToByteArray(pulseValues.getPulseInterval());
		byte[] frequencyByte = MDBUtils.intToByteArray(pulseValues.getPulseFrequency());
		dataBytes = MDBUtils.connectBytes(dataBytes, voltageByte, durationByte, intervalByte, frequencyByte);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST, dataBytes);
	}

	public static <T> byte[] getRequestSetPulseDuration(T... args) {
		int duration = (int) args[0];
		byte[] dataBytes = {MDBUtils.MDB_CMD_SET_PULSE, MDBUtils.MDB_CMD_SET_PULSE_DURATION};
		byte[] durationByte = MDBUtils.intToByteArray(duration);
		dataBytes = MDBUtils.connectBytes(dataBytes, durationByte);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST,dataBytes);
	}

	public static <T> byte[] getRequestSetPulseVoltage(T... args) {
		int voltage = (int) args[0];
		byte[] dataBytes = {MDBUtils.MDB_CMD_SET_PULSE, MDBUtils.MDB_CMD_SET_PULSE_VOLTAGE};
		byte[] voltageByte = MDBUtils.intToByteArray(voltage);
		dataBytes = MDBUtils.connectBytes(dataBytes, voltageByte);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST,dataBytes);
	}

	public static <T> byte[] getRequestEnterFactoryMode(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_SET_PULSE, MDBUtils.MDB_CMD_FACTORY_MODE};
		byte[] factoryModeData = MDBUtils.intToByteArray(1);
		dataBytes = MDBUtils.connectBytes(dataBytes, factoryModeData);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST,dataBytes);
	}

	public static <T> byte[] getRequestQuitFactoryMode(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_SET_PULSE, MDBUtils.MDB_CMD_FACTORY_MODE};
		byte[] factoryModeData = MDBUtils.intToByteArray(0);
		dataBytes = MDBUtils.connectBytes(dataBytes, factoryModeData);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST,dataBytes);
	}

	public static <T> byte[] getRequestHardwareDiagnoseTest(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_HARDWARE_DIAGNOSE_TEST};
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST,dataBytes);
	}

	public static <T> byte[] getRequestVersionRequest(T... args) {
		return MDBUtils.MDB_CMD_VERSION_REQUEST;
	}

	public static <T> byte[] getRequestHardwareVersionRequest(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_HARDWARE_VERSION_REQUEST};
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST,dataBytes);
	}

	public static <T> byte[] getRequestReset(T... args) {
		byte[] dataBytes = {MDBUtils.MDB_CMD_RESET};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST,dataBytes);
	}

	public static <T> byte[] getRequestSetupConfig(T... args) {
		byte[] dataBytes = {0x11, 0x00, 0x03, 0x00, 0x00, 0x01};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST, dataBytes);
	}

	public static <T> byte[] getRequestSetupPrice(T... args) {
		byte[] dataBytes = {0x11, 0x01, 0x2b, 0x5c, 0x00, 0x0a};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST, dataBytes);
	}

	public static <T> byte[] getRequestExpansionRequestID(T... args) {
		byte[] dataBytes = {0x17, 0x00, 0x49, 0x44, 0x53, 0x30, 0x30, 0x30,
							0x30, 0x31, 0x30, 0x30, 0x30, 0x36, 0x36, 0x30,
							0x32, 0x49, 0x44, 0x53, 0x4d, 0x44, 0x42, 0x56,
							0x4d, 0x43, 0x43, 0x4f, 0x4e, 0x30, 0x32};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST, dataBytes);
	}

	public static <T> byte[] getRequestExpansionEnableOptions(T... args) {
		byte[] dataBytes = {0x17, 0x04, 0x00, 0x00, 0x00, 0x3E};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST, dataBytes);
	}

	public static <T> byte[] getRequestExpansionEnableReader(T... args) {
		byte[] dataBytes = {0x14, 0x01};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST, dataBytes);
	}

	public static <T> byte[] getRequestBeginSession(T... args) {
		MDBValues mdbValues = (MDBValues) args[0];
		byte[] dataBytes = {MDBUtils.MDB_CMD_BEGIN_SESSION};
		byte[] balanceAvailable = new byte[2];
		if(mdbValues.getBalance() < 0) {
			balanceAvailable = new byte[]{0x03, (byte) 0xE8};//funds/balance Available
		} else {
			balanceAvailable = new byte[]{(byte)(mdbValues.getBalance() >> 8 & 0xFF), (byte)(mdbValues.getBalance() & 0xFF)};//funds/balance Available
		}
		byte[] paymentMediaID = {(byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF};//Payment Media ID
		byte[] paymentType = {0x00};//Payment Type
		byte[] paymentData = {0x00, 0x00};//Payment Data
		dataBytes = MDBUtils.connectBytes(dataBytes, balanceAvailable, paymentMediaID, paymentType, paymentData);
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST, dataBytes);
	}

	public static <T> byte[] getRequestVendRequest(T... args) {
		byte[] dataBytes = {0x13, 0x00, 0x00, 0x64, 0x00, (byte)0xe1};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST, dataBytes);
	}

	public static <T> byte[] getRequestVendDispatch(T... args) {
		byte[] dataBytes = {0x13, 0x02, 0x00, (byte)0xe1};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST, dataBytes);
	}

	public static <T> byte[] getRequestEndSession(T... args) {
		byte[] dataBytes = {0x13, 0x04};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST, dataBytes);
	}

	public static <T> byte[] getRequestNegativeVendApproved(T... args) {
		byte[] dataBytes = {0x13, 0x06, 0x00, 0x00, 0x00, 0x64, (byte)0xFF, (byte)0xFF};
		dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
		return MDBUtils.mergePacket(MDBUtils.MDB_CMD_COMMON_REQUEST, dataBytes);
	}

}
