package com.wizarpos.util;

import java.util.Arrays;

public class MDBUtils {

	public static final int BLACK_LOG = 1;
	public static final int RED_LOG = 2;
	public static final int BLUE_LOG = 3;
	public static final int GREEN_LOG = 4;
	public static final int DIALOG_CONFIRM = 8;
	public static final int DIALOG_SET_PULSE = 9;
	public static final int DIALOG_ITEM = 10;
	public static final int DIALOG_READCARD = 11;
	public static final int ENABLE_ALL_UI = 15;
	public static final int DISABLE_ALL_UI = 16;

	public static final int TYPE_PULSE_INTERVAL = 20;
	public static final int TYPE_PULSE_FREQUENCY = 21;
	public static final int TYPE_PULSE_DURATION = 22;
	public static final int TYPE_PULSE_VOLTAGE = 23;
	public static final int TYPE_BALANCE = 24;
	public static final int TYPE_SPN_FIRMWARE_ITEM = 25;
	public static final int TYPE_SPN_MDB_LEVEL = 26;
	public static final int TYPE_CB_CHECK_ALWAYS_IDLE = 27;
	public static final int TYPE_CB_CHECK_32BIT_MONETARY = 28;
	public static final int TYPE_CB_CHECK_NEGATIVE_VEND = 29;

	public static final int TEST_MSG = 99;

	public static final byte MDB_CMD_COMMON_RESPONSE = 0x01;
	public static final byte MDB_CMD_COMMON_REQUEST = 0x00;
	public static final byte MDB_CMD_RESET = 0x00;
	public static final byte MDB_CMD_READER_CONFIG = 0x01;
	public static final byte MDB_CMD_EXPANSION_PERIPHERALID = 0x09;
	public static final byte MDB_CMD_EXPANSION_ENABLE_OPTIONS = 0x17;
	public static final byte MDB_CMD_REVALUE_APPROVED = 0x0D;
	public static final byte MDB_CMD_REVALUE_LIMIT_AMOUNT = 0x0F;
	public static final byte MDB_CMD_END_SESSION = 0x07;
	public static final byte MDB_CMD_VEND_APPROVED = 0x05;
	public static final byte MDB_CMD_VEND_DENIED = 0x06;
	public static final byte MDB_CMD_BEGIN_SESSION = 0x03;
	public static final byte MDB_CMD_READER_CANCEL = 0x08;
	public static final byte MDB_CMD_TRIGGER_PULSE = 0x20;
	public static final byte MDB_CMD_TRIGGER_PULSE_ONESHOT = 0x21;
	public static final byte MDB_CMD_SET_PULSE = (byte)0x90;
	public static final byte MDB_CMD_SET_PULSE_DURATION = 0x03;
	public static final byte MDB_CMD_SET_PULSE_VOLTAGE = 0x04;
	public static final byte MDB_CMD_FACTORY_MODE = 0x05;
	public static final byte MDB_CMD_HARDWARE_DIAGNOSE_TEST = (byte)0x94;
	public static final byte MDB_CMD_HARDWARE_VERSION_REQUEST = (byte)0x93;
	public static final byte MDB_CMD_NEGATIVE_VEND_APPROVED = 0x05;
	public static final byte[] MDB_CMD_VERSION_REQUEST = {0x09, 0x03, 0x00, (byte) 0x92, (byte) 0x6E, (byte) 0x0D};

	public static byte[] addMdbCheckSum(byte[] buf) {
		byte[] ret = connectBytes(buf, new byte[]{0x00});
		for(int i = 0; i < ret.length-1; i++){
			ret[ret.length-1] += ret[i];
		}
		return ret;
	}

	/*
	 * add 09 before data,add checksum and 0D after length and mode
	 * data: data content
	 * mode: 00: request, 01: response
	 */
	public static byte[] mergePacket(int mode, byte[] data) {
		byte startCode = 0x09;
		byte endCode = 0x0d;
		byte modebyte = 0x00;
		if (mode == 1) {
			modebyte = 0x01;
		}
		//------
		byte[] modeANDdata = new byte[1 + data.length];
		modeANDdata[0] = modebyte;
		System.arraycopy(data, 0, modeANDdata, 1, data.length);
		byte checkSumLRC = getLRC(modeANDdata);
		//------
		byte[] mergeBytes = new byte[1 + 1 + 1 + data.length + 1 + 1];
		mergeBytes[0] = startCode;
		mergeBytes[1] = (byte) (1 + 1 + data.length);
		mergeBytes[2] = modebyte;
		System.arraycopy(data, 0, mergeBytes, 3, data.length);
		mergeBytes[mergeBytes.length - 2] = checkSumLRC;
		mergeBytes[mergeBytes.length - 1] = endCode;
		//String.format("%02X", byte)
		return mergeBytes;
	}

	public static byte getLRC(byte[] buf) {
		int rr = 0x00;
		for (byte b : buf) {
			rr += (b & 0xFF);
		}
		rr = ~rr + 1;
		return (byte) rr;
	}

	/**
	 * split byte[]
	 */
	public static byte[][] splitBytes(byte[] bytes, int size) {
		double splitLength = Double.parseDouble(size + "");
		int arrayLength = (int) Math.ceil(bytes.length / splitLength);
		byte[][] result = new byte[arrayLength][];
		int from, to;
		for (int i = 0; i < arrayLength; i++) {

			from = (int) (i * splitLength);
			to = (int) (from + splitLength);
			if (to > bytes.length)
				to = bytes.length;
			result[i] = Arrays.copyOfRange(bytes, from, to);
		}
		return result;
	}

	public static byte[] intToByteArray(int value) {
		return new byte[] {
				(byte) (value & 0xFF),
				(byte) ((value >> 8) & 0xFF),
				(byte) ((value >> 16) & 0xFF),
				(byte) ((value >> 24) & 0xFF)
		};
	}

	public static byte[] connectBytes(byte[]... bytes) {
		int length = 0;
		for (byte[] b : bytes) {
			length += b.length;
		}
		byte[] result = new byte[length];
		int pos = 0;
		for (byte[] b : bytes) {
			System.arraycopy(b, 0, result, pos, b.length);
			pos += b.length;
		}
		return result;
	}
}
